/* Videod3d9.cpp
 *
 * The benchmark is run via the message processing loop in WinMain
 * after Windows is created, variables assigned, command line
 * commands evaluated (see runVidD3D9_64.bat), log file opened,
 * and configuration details obtained (GetDetails). Message processing
 * phases to run the six different tests are:
 *
 * Phase 1 - InitD3D(), InitGeometry() + SetupLights(), InitVB() - tests >2,
 *
 * Phase 2 - Render() for limitsecs seconds, first SetupMatrices()
 *
 * Phase 3 - Calculate speeds, log results, Cleanup()
 *
 * Phase 4 - Save configuration details, Exit  
 *
 * Test 1 - Rotating Egg Gouraud shading
 * Test 2 - Rotating Egg Wireframe, Vsync (monitor refresh rate)
 * Test 3 - Rotating Egg and 500 Cubes Wireframe
 * Test 4 - Textured Tunnel and Roataing Objects
 * Test 5 - Plain Colour Objects, different rotations
 * Test 6 - As Test 5 but textured
 * Test 7 - As Test 6 but added shading from Pixel Shader 2.0
 * Test 8 - As Test 6 but added code from Vertex Shader 2.0
 *
 * InitD3D() - Create the D3D object and D3DDevice plus setup and some logging.
 * Check for HW support of Version 2.0 of Pixel Shader and Vertex Shader
 *
 * InitVB() - create vertex arrays for cubes, different according to testToRun,
 * use functions such as D3DXCreateTeapot and D3DXCreateTorus with texture 
 * settings, CreateTextureFromFile imageFiles[x], D3DXCreateEffectFromFile
 * OneEffect.fx, pixel.shd and vertex.shd
 *
 * InitGeometry() - D3DXLoadMeshFromX egg.x, texture settings 
 *
 * SetupMatrices() - testToRun 1-3 and 4-6 different views 
 *  
 * Render()
 * testToRun 1-3  DrawSubset egg,
 * testToRun   3  DrawPrimitive 500 cubes,
 * testToRun   4  DrawPrimitive tunnel sides and top/bottom, renderSphere(),
 *                renderCylinder(), renderTorus(), renderCube1() to renderCube4()
 * testToRun 5-8  3 x renderCube1(), 3 x renderCube2(), 4 x renderCube3()
 *                4 x renderCube4(), 2 x renderSphere(), 2 x renderTeaPot(),
 *                2 x renderTorus(), renderPolygon() background, 2 x renderCylinder(),
 *                2 x renderBox(), renderText().  
 * ID3DXEffect    Used in tests 4 and 6 to 8 with DrawSubset 
 * SetTechnique   Used for tests 7 and 8 with parameters via SetFloatArray and
 *                SetFloat. Test 8 uses SetMatrix to enable movement parameters
 *  
 * getDetails() for 32 or 64 bit compilation need CPUasm.obj, asmtime.obj
 * and asmtime.h compiled from code in:
 * http://homepage.virgin.net/roy.longbottom/newsource.zip
 *
 * Some from examples Copyright (c) Microsoft Corporation

 *************************************************************************/

#include <Windows.h>
#include <mmsystem.h>
#include <time.h>
#include <d3dx9.h>
#include "asmtime.h"
#pragma warning( disable : 4996 ) // disable deprecated warning 
#include <strsafe.h>
#pragma warning( default : 4996 ) 


INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT );
LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
HRESULT InitD3D( HWND hWnd );
HRESULT InitGeometry();
HRESULT InitVB();

VOID SetupMatrices();
VOID SetupLights();
VOID Render();
VOID Cleanup();
void local_time();

void renderCube1(float, float, float);
void renderCube2(float, float, float);
void renderCube3(float, float, float);
void renderCube4(float, float, float);
void renderSphere(float, float, float);
void renderTeaPot(float, float, float);
void renderTorus(float, float, float);
void renderPolygon(float, float, float);
void renderCylinder(float, float, float);
void renderBox(float, float, float);
void renderText(float, float, float);

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
LPDIRECT3D9             g_pD3D           = NULL; // Used to create the D3DDevice
LPDIRECT3DDEVICE9       g_pd3dDevice     = NULL; // Our rendering device

LPD3DXMESH              g_pMesh          = NULL; // Our mesh object in sysmem
LPD3DXMESH              g_SphereMesh     = NULL;
LPD3DXMESH              g_TeaPotMesh     = NULL;
LPD3DXMESH              g_TorusMesh      = NULL;
LPD3DXMESH              g_PolygonMesh    = NULL;
LPD3DXMESH              g_CylinderMesh   = NULL;
LPD3DXMESH              g_BoxMesh        = NULL;
LPD3DXMESH              g_TextMesh       = NULL;
ID3DXEffect*            g_pEffect        = NULL;
ID3DXEffect*            g_pEffect2       = NULL;
LPDIRECT3DTEXTURE9      g_pTexture_0     = NULL;
LPDIRECT3DTEXTURE9      g_pTexture_1     = NULL;
LPDIRECT3DTEXTURE9      g_pTexture_2     = NULL;
LPDIRECT3DTEXTURE9      g_pTexture_3     = NULL;
LPDIRECT3DTEXTURE9      g_pTexture_4     = NULL;
D3DXVECTOR3             vEyePt2;

D3DMATERIAL9*           g_pMeshMaterials = NULL; // Materials for our mesh
LPDIRECT3DTEXTURE9*     g_pMeshTextures  = NULL; // Textures for our mesh
DWORD                   g_dwNumMaterials = 0L;   // Number of mesh materials

LPDIRECT3DVERTEXBUFFER9 g_pVB1           = NULL; // Buffer to hold vertices 500 cubes
LPDIRECT3DVERTEXBUFFER9 g_pVB2           = NULL; // 3 Cubes different colours
LPDIRECT3DVERTEXBUFFER9 g_pVB3           = NULL; // Cube for texturing 
LPDIRECT3DVERTEXBUFFER9 g_pVB4a          = NULL; // Tunnel top/bottom
LPDIRECT3DVERTEXBUFFER9 g_pVB4b          = NULL; // Tunnel sides 

D3DXMATRIXA16           matProj2;
D3DXMATRIXA16           matView2;


// Structures for our custom vertex types

struct CUSTOMVERTEX
{
    float x, y, z;  // The transformed(screen space) position for the vertex.
    DWORD colour;        // The vertex colour.
};

struct CUSTOMVERTEX2
{
    float x, y, z;
    float tu, tv;
};


// Our custom FVFs, which describe our custom vertex structures
#define D3DFVF_CUSTOMVERTEX (D3DFVF_XYZ|D3DFVF_DIFFUSE)
#define D3DFVF_CUSTOMVERTEX2 ( D3DFVF_XYZ | D3DFVF_TEX1 )


const int g_cube_copies = 500;
const int g_cube_count=6 * 2 * g_cube_copies; // 6 faces, each face is 2 triangles
const int g_cube_count2 = 12;

int       g_tunnel_count;
int       phase = 1;
int       testToRun = 1;
int       lastTest  = 8; 
int       vertSize5;
int       rotPos;
int       keyPress;
int       pixelsWide;
int       pixelsHigh;
int       bpp;
int       flip = 0;

float     vEyeOut = -7.0f;
float     vEyeIn  = 3.0f;
float     vEyeInc = 0.005f;
float     rotateInc;
float     rotateAdj;
float     rotateCount[100];
float     aspect;
float     aspc;
float     pixVary = 1.0;
float     pv[4];

double    limitsecs = 10.0;
double    passSecsLimit = 60.0;
double    fps = 1.0;
double    Passfps;
double    results[12];


char      testTitle[12][80] = {" "};
char      thisAdapter[100] = {"                  "};
char      commandLine[512] = "None";
char      logFile[100] = "VideoLog.txt";
char      timeday[32];
char      version[100] = "64 Bit Version 1.1";
char      imageFiles[5][50];

BOOL    autoRun = FALSE;
BOOL    escPress = FALSE;
BOOL    saveHeadings = TRUE;
BOOL    saveConfig = TRUE;
BOOL    starting = TRUE;
BOOL    runOne = FALSE;
BOOL    pixelShaderV2 = TRUE;
BOOL    vertexShaderV2 = TRUE;

FILE    *outfile = NULL;


VOID SetupLights()
{
    D3DXVECTOR3 vecDir;
    D3DLIGHT9 light;
    ZeroMemory( &light, sizeof(D3DLIGHT9) );
    light.Type       = D3DLIGHT_DIRECTIONAL;
    light.Diffuse.r  = 1.0f;
    light.Diffuse.g  = 1.0f;
    light.Diffuse.b  = 1.0f;
    vecDir = D3DXVECTOR3(cosf(timeGetTime()/350.0f),
                         1.0f,
                         sinf(timeGetTime()/350.0f) );
    D3DXVec3Normalize( (D3DXVECTOR3*)&light.Direction, &vecDir );
    light.Range       = 1000.0f;
    g_pd3dDevice->SetLight( 0, &light );
    g_pd3dDevice->LightEnable( 0, TRUE );
    g_pd3dDevice->SetRenderState( D3DRS_LIGHTING, TRUE );

    // Finally, turn on some ambient light.
    if (testToRun == 2)
    {
        g_pd3dDevice->SetRenderState( D3DRS_AMBIENT, 0x00ffffff );  // wireframe
    }
    else
    {
        g_pd3dDevice->SetRenderState( D3DRS_AMBIENT, 0x00203020 ); // solid
    }    
}


HRESULT InitD3D( HWND hWnd )
{

    // Create the D3D object.
    if( NULL == ( g_pD3D = Direct3DCreate9( D3D_SDK_VERSION ) ) )
    {
        MessageBox(NULL, "Direct3DCreate9 Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n Direct3DCreate9 Failed\n\n");
        return E_FAIL;
    }

    D3DCAPS9 d3dcaps9;
    ZeroMemory(&d3dcaps9, sizeof(d3dcaps9));

    HRESULT hRes = g_pD3D->GetDeviceCaps(D3DADAPTER_DEFAULT,
                                        D3DDEVTYPE_HAL,
                                        &d3dcaps9);
    
    if (FAILED(hRes))
    {
        MessageBox(NULL, "GetDeviceCaps Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n GetDeviceCaps Failed\n\n");
        return E_FAIL;
    } 

    // Check for pixel shader version 2.0 support.
    if (d3dcaps9.PixelShaderVersion < D3DPS_VERSION(2,0))
    {
        pixelShaderV2 = FALSE;
    }
    // Check for vertex shader version 2.0 support.
    if (d3dcaps9.VertexShaderVersion < D3DVS_VERSION(2,0))
    {
        vertexShaderV2 = FALSE;
    }


    // Set up the structure used to create the D3DDevice.
    D3DPRESENT_PARAMETERS d3dpp; 
    ZeroMemory( &d3dpp, sizeof(d3dpp) );
    d3dpp.Windowed = TRUE;
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;
    d3dpp.EnableAutoDepthStencil = TRUE;
    d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
    if (testToRun == 2)
    {
        d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_ONE; // vsync
    }
    else
    {
        d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;
    }
    // Create the D3DDevice
    if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
                                      D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                      &d3dpp, &g_pd3dDevice ) ) )
    {
        MessageBox(NULL, "CreateDevice Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n CreateDevice Failed\n\n");
        return E_FAIL;
    }
    D3DADAPTER_IDENTIFIER9 id;
    g_pD3D->GetAdapterIdentifier( D3DADAPTER_DEFAULT , 0, &id );
    StringCchPrintf(thisAdapter, 40, " %s", id.Description);

    if (starting)
    {
        if (testToRun == lastTest || testToRun > 1) runOne = TRUE;
        if (runOne)
        {
            fprintf (outfile, " %s at %d x %d x %d bits\n", testTitle[testToRun], pixelsWide, pixelsHigh, bpp);
        }
        else
        {
            if (saveHeadings)
            {
                fprintf(outfile, "               ..................... Frames Per Second ......................\n"); 
                fprintf(outfile, " Resolution    Shaded WireEgg     500 Texture  Colour Texture   Pixel  Vertex\n");
                fprintf(outfile, "                  Egg   Vsync   Cubes  Tunnel Objects Objects Shader2 Shader2\n\n");
            }
            fprintf (outfile, "%5d %4d %2d ", pixelsWide, pixelsHigh, bpp);
        }
        starting = FALSE;
    }
    // Turn on the zbuffer
    g_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );

    // Set to Gouraud shading. This is the default for Direct3D.
    if(FAILED(g_pd3dDevice->SetRenderState(D3DRS_SHADEMODE,
                                D3DSHADE_GOURAUD)))
    {
            MessageBox(NULL, "SetRenderState  1 Failed", "ERROR", MB_OK);
            fprintf(outfile, "\n SetRenderState 1 Failed\n\n");
            return E_FAIL;
    }
    if (testToRun == 2 || testToRun == 3)
    {
        if(FAILED(g_pd3dDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME)))
        {
                MessageBox(NULL, "SetRenderState 2 Failed", "ERROR", MB_OK);
                fprintf(outfile, "\n SetRenderState 2 Failed\n\n");
                return E_FAIL;
        }
    }
    else
    {
        if(FAILED(g_pd3dDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID)))
        {
                MessageBox(NULL, "SetRenderState 3 Failed", "ERROR", MB_OK);
                fprintf(outfile, "\n SetRenderState 3 Failed\n\n");
                return E_FAIL;
        }
    }

    // Setup the lights and materials
    SetupLights();
    g_pd3dDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE);

    return S_OK;
} // InitD3D

HRESULT InitVB()
{
    char     amsg[32] = "";

    int vertSize;
    int i, j, k, l;
    int nNumVerts;    

    int incCopies;
    int theseCopies = 0;

    float xx1;
    float mult;
    
    //  500 Cubes
    float xv[10] = {1.00f, 1.08f, 1.16f, 1.22f, 1.28f, 1.33f, 1.37f, 1.39f, 1.41f, 1.41f};
    float yv[10] = {1.00f, 0.91f, 0.81f, 0.71f, 0.60f, 0.48f, 0.37f, 0.25f, 0.12f, 0.00f};    
    DWORD  cl[7] = {0xFF0000FF, 0xFF00FFFF, 0xFF00FF00, 0xFFFFFF00, 0xFFFF0000, 0xFFFF00FF, 0xFFFFFFFF};

    CUSTOMVERTEX vertices2[] =
    {
          //Front face
       {-1.37f,-0.37f,-1.0f,0xFF0000FF},{-0.37f, 1.37f,-1.0f,0xFF0000FF},{ 1.37f, 0.37f,-1.0f,0xFF0000FF},
       { 1.37f, 0.37f,-1.0f,0xFF0000FF},{ 0.37f,-1.37f,-1.0f,0xFF0000FF},{-1.37f,-0.37f,-1.0f,0xFF0000FF},
          //Back face
       { 0.37f,-1.37f, 1.0f,0xFF0000FF},{ 1.37f, 0.37f, 1.0f,0xFF0000FF},{-0.37f, 1.37f, 1.0f,0xFF0000FF},
       {-0.37f, 1.37f, 1.0f,0xFF0000FF},{-1.37f,-0.37f, 1.0f,0xFF0000FF},{ 0.37f,-1.37f, 1.0f,0xFF0000FF},
          //Top face
       {-0.37f, 1.37f,-1.0f,0xFF0000FF},{-0.37f, 1.37f, 1.0f,0xFF0000FF},{ 1.37f, 0.37f, 1.0f,0xFF0000FF},
       { 1.37f, 0.37f, 1.0f,0xFF0000FF},{ 1.37f, 0.37f,-1.0f,0xFF0000FF},{-0.37f, 1.37f,-1.0f,0xFF0000FF},
    
          //Bottom face
       { 0.37f,-1.37f,-1.0f,0xFF0000FF},{ 0.37f,-1.37f, 1.0f,0xFF0000FF},{-1.37f,-0.37f, 1.0f,0xFF0000FF},
       {-1.37f,-0.37f, 1.0f,0xFF0000FF},{-1.37f,-0.37f,-1.0f,0xFF0000FF},{ 0.37f,-1.37f,-1.0f,0xFF0000FF},
    
          //Left face
       {-1.37f,-0.37f, 1.0f,0xFF0000FF},{-0.37f, 1.37f, 1.0f,0xFF0000FF},{-0.37f, 1.37f,-1.0f,0xFF0000FF},
       {-0.37f, 1.37f,-1.0f,0xFF0000FF},{-1.37f,-0.37f,-1.0f,0xFF0000FF},{-1.37f,-0.37f, 1.0f,0xFF0000FF},
    
          //Right face
       { 0.37f,-1.37f,-1.0f,0xFF0000FF},{ 1.37f, 0.37f,-1.0f,0xFF0000FF},{ 1.37f, 0.37f, 1.0f,0xFF0000FF},
       { 1.37f, 0.37f, 1.0f,0xFF0000FF},{ 0.37f,-1.37f, 1.0f,0xFF0000FF},{ 0.37f,-1.37f,-1.0f,0xFF0000FF},
    };
    
    CUSTOMVERTEX vertices3[] =
    {
      //Front face
       {-1.4f, 0.0f,-1.0f,0xFF0000FF},{ 0.0f, 1.4f,-1.0f,0xFF0000FF},{ 1.4f, 0.0f,-1.0f,0xFF0000FF},
       { 1.4f, 0.0f,-1.0f,0xFF0000FF},{ 0.0f,-1.4f,-1.0f,0xFF0000FF},{-1.4f, 0.0f,-1.0f,0xFF0000FF},
    
          //Back face
       { 0.0f,-1.4f, 1.0f,0xFF0000FF},{ 1.4f, 0.0f, 1.0f,0xFF0000FF},{ 0.0f, 1.4f, 1.0f,0xFF0000FF},
       { 0.0f, 1.4f, 1.0f,0xFF0000FF},{-1.4f, 0.0f, 1.0f,0xFF0000FF},{ 0.0f,-1.4f, 1.0f,0xFF0000FF},
    
          //Top face
       { 0.0f, 1.4f,-1.0f,0xFF0000FF},{ 0.0f, 1.4f, 1.0f,0xFF0000FF},{ 1.4f, 0.0f, 1.0f,0xFF0000FF},
       { 1.4f, 0.0f, 1.0f,0xFF0000FF},{ 1.4f, 0.0f,-1.0f,0xFF0000FF},{ 0.0f, 1.4f,-1.0f,0xFF0000FF},
    
          //Bottom face
       { 0.0f,-1.4f,-1.0f,0xFF0000FF},{ 0.0f,-1.4f, 1.0f,0xFF0000FF},{-1.4f, 0.0f, 1.0f,0xFF0000FF},
       {-1.4f, 0.0f, 1.0f,0xFF0000FF},{-1.4f, 0.0f,-1.0f,0xFF0000FF},{ 0.0f,-1.4f,-1.0f,0xFF0000FF},
    
          //Left face
       {-1.4f, 0.0f, 1.0f,0xFF0000FF},{ 0.0f, 1.4f, 1.0f,0xFF0000FF},{ 0.0f, 1.4f,-1.0f,0xFF0000FF},
       { 0.0f, 1.4f,-1.0f,0xFF0000FF},{-1.4f, 0.0f,-1.0f,0xFF0000FF},{-1.4f, 0.0f, 1.0f,0xFF0000FF},
    
          //Right face
       { 0.0f,-1.4f,-1.0f,0xFF0000FF},{ 1.4f, 0.0f,-1.0f,0xFF0000FF},{ 1.4f, 0.0f, 1.0f,0xFF0000FF},
       { 1.4f, 0.0f, 1.0f,0xFF0000FF},{ 0.0f,-1.4f, 1.0f,0xFF0000FF},{ 0.0f,-1.4f,-1.0f,0xFF0000FF},
    };
     
    CUSTOMVERTEX vertices[36];

    vertSize = sizeof(CUSTOMVERTEX) * 36;

    // Create the vertex buffer.
    if( FAILED( g_pd3dDevice->CreateVertexBuffer( vertSize * g_cube_copies,
                                                  D3DUSAGE_WRITEONLY,
                                                  D3DFVF_CUSTOMVERTEX,
                                                  D3DPOOL_DEFAULT,
                                                  &g_pVB1,
                                                  NULL ) ) )
    {
        MessageBox(NULL, "CreateVertexBuffer 1 Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n CreateVertexBuffer 1 Failed\n\n");
        return E_FAIL;
    }

    char* pVerticesa;

    if( FAILED( g_pVB1->Lock( 0, vertSize*g_cube_copies, (void**)&pVerticesa, 0 ) ) )
    {
        MessageBox(NULL, "Lock 1 Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n Lock 1 Failed\n\n");
        return E_FAIL;
    }

    incCopies =  25;
    l = 0;
    mult = 1.3f;

    for (k=0; k<g_cube_copies / incCopies / 2; k++)
    {
        for (i=0; i<36; i++)
        {
            if (vertices2[i].x ==  1.37f) vertices[i].x =  xv[k] * mult;
            if (vertices2[i].x == -1.37f) vertices[i].x = -xv[k] * mult;
            if (vertices2[i].y ==  1.37f) vertices[i].y =  xv[k] * mult;
            if (vertices2[i].y == -1.37f) vertices[i].y = -xv[k] * mult;
            if (vertices2[i].x ==  0.37f) vertices[i].x =  yv[k] * mult;
            if (vertices2[i].x == -0.37f) vertices[i].x = -yv[k] * mult;
            if (vertices2[i].y ==  0.37f) vertices[i].y =  yv[k] * mult;
            if (vertices2[i].y == -0.37f) vertices[i].y = -yv[k] * mult;
            vertices3[i].z = vertices2[i].z * mult;
        }
        xx1 = 0.0f;
        for (i=theseCopies; i<theseCopies + incCopies; i++) 
        {
            for (j=0; j<36; j++)
            {
                vertices3[j].x = vertices[j].x + xx1;
                vertices3[j].y = vertices[j].y;
                vertices3[j].colour = cl[l];
                l = l + 1;
                if (l == 7) l = 0;
            }
            memcpy( pVerticesa + i * vertSize, vertices3, vertSize );
            xx1 = xx1 + 0.02f;
        }
        theseCopies = theseCopies + incCopies;
        xx1 = 0.0f;
        for (i=theseCopies; i<theseCopies + incCopies; i++)
        {
            for (j=0; j<36; j++)
            {
                vertices3[j].x = vertices[j].x - xx1;
                vertices3[j].y = vertices[j].y;
                vertices3[j].colour = cl[l];
                l = l + 1;
                if (l == 7) l = 0;
            }
            memcpy( pVerticesa + i * vertSize, vertices3, vertSize );
            xx1 = xx1 + 0.02f;
        }
        theseCopies = theseCopies + incCopies;
    }
    g_pVB1->Unlock();

    //  testToRun > 3

    LPD3DXMESH tempMesh   = NULL;
    D3DXCreateSphere(g_pd3dDevice, 1.8f, 50, 50, &tempMesh, NULL);
    if (testToRun == 4 || testToRun > 5)
    {
        tempMesh->CloneMeshFVF( 0, D3DFVF_CUSTOMVERTEX2, g_pd3dDevice, &g_SphereMesh );
    }
    else
    {
        tempMesh->CloneMeshFVF( 0, D3DFVF_CUSTOMVERTEX, g_pd3dDevice, &g_SphereMesh );
    }
    tempMesh->Release();

    LPDIRECT3DVERTEXBUFFER9 pTempVertexBuffer;

    if( SUCCEEDED( g_SphereMesh->GetVertexBuffer( &pTempVertexBuffer ) ) )
    {
        nNumVerts = g_SphereMesh->GetNumVertices();
        if (testToRun == 4 || testToRun > 5)
        {
            CUSTOMVERTEX2 *pVertices1a = NULL;
            pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices1a, 0 );
            for(i=0; i<nNumVerts-3; i=i+4)
            {
                pVertices1a[i].tu   = 0.0f;
                pVertices1a[i].tv   = 0.0f;
                pVertices1a[i+1].tu = 1.0f;
                pVertices1a[i+1].tv = 0.0f;
                pVertices1a[i+2].tu = 0.0f;
                pVertices1a[i+2].tv = 1.0f;
                pVertices1a[i+3].tu = 1.0f;
                pVertices1a[i+3].tv = 1.0f;
            }
        }
        else
        {
            CUSTOMVERTEX *pVertices1 = NULL;
            pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices1, 0 );
            for(i=0; i<nNumVerts-1; i=i+2)
            {
                pVertices1[i].colour = D3DCOLOR_COLORVALUE( 1.0f , 1.0f, 0.0f, 1.0f);
                pVertices1[i+1].colour = D3DCOLOR_COLORVALUE( 1.0f , 0.0f, 0.0f, 1.0f);
            }
        }
        pTempVertexBuffer->Unlock();
        pTempVertexBuffer->Release();
    }
    else
    {
        MessageBox(NULL, "GetVertexBuffer 1 Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n GetVertexBuffer 1 Failed\n\n");
        return E_FAIL;
    }

    tempMesh   = NULL;
    D3DXCreateTeapot(g_pd3dDevice, &tempMesh, NULL);
    if (testToRun == 4 || testToRun > 5)
    {
        tempMesh->CloneMeshFVF( 0, D3DFVF_CUSTOMVERTEX2, g_pd3dDevice, &g_TeaPotMesh );
    }
    else
    {
        tempMesh->CloneMeshFVF( 0, D3DFVF_CUSTOMVERTEX, g_pd3dDevice, &g_TeaPotMesh );
    }
    tempMesh->Release();

    if( SUCCEEDED( g_TeaPotMesh->GetVertexBuffer( &pTempVertexBuffer ) ) )
    {
        nNumVerts = g_TeaPotMesh->GetNumVertices();
        if (testToRun == 4 || testToRun > 5)
        {
            CUSTOMVERTEX2 *pVertices2a = NULL;
            pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices2a, 0 );
            for(i=0; i<nNumVerts-3; i=i+4)
            {
                pVertices2a[i].tu   = 0.0f;
                pVertices2a[i].tv   = 0.0f;
                pVertices2a[i+1].tu = 1.0f;
                pVertices2a[i+1].tv = 0.0f;
                pVertices2a[i+2].tu = 0.0f;
                pVertices2a[i+2].tv = 1.0f;
                pVertices2a[i+3].tu = 1.0f;
                pVertices2a[i+3].tv = 1.0f;
            }
        }
        else
        {
            CUSTOMVERTEX *pVertices2 = NULL;
            pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices2, 0 );            
            for(i=0; i<nNumVerts-1; i=i+2)
            {
                pVertices2[i].colour = D3DCOLOR_COLORVALUE( 0.0f , 1.0f, 1.0f, 1.0f);
                pVertices2[i+1].colour = D3DCOLOR_COLORVALUE( 0.0f , 0.0f, 1.0f, 1.0f);
            }
        }
        pTempVertexBuffer->Unlock();
        pTempVertexBuffer->Release();
    }
    else
    {
        MessageBox(NULL, "GetVertexBuffer 2 Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n GetVertexBuffer 2 Failed\n\n");
        return E_FAIL;
    }

    tempMesh   = NULL;
    D3DXCreateTorus(g_pd3dDevice, 1.0f, 2.0f, 30, 30, &tempMesh, NULL);
    if (testToRun == 4 || testToRun > 5)
    {
        tempMesh->CloneMeshFVF( 0, D3DFVF_CUSTOMVERTEX2, g_pd3dDevice, &g_TorusMesh );
    }
    else
    {
        tempMesh->CloneMeshFVF( 0, D3DFVF_CUSTOMVERTEX, g_pd3dDevice, &g_TorusMesh );
    }
    tempMesh->Release();

    if( SUCCEEDED( g_TorusMesh->GetVertexBuffer( &pTempVertexBuffer ) ) )
    {
        nNumVerts = g_TorusMesh->GetNumVertices();
        if (testToRun == 4 || testToRun > 5)
        {
            CUSTOMVERTEX2 *pVertices3a = NULL;
            pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices3a, 0 );
            for(i=0; i<nNumVerts-3; i=i+4)
            {
                pVertices3a[i].tu   = 0.0f;
                pVertices3a[i].tv   = 0.0f;
                pVertices3a[i+1].tu = 1.0f;
                pVertices3a[i+1].tv = 0.0f;
                pVertices3a[i+2].tu = 0.0f;
                pVertices3a[i+2].tv = 1.0f;
                pVertices3a[i+3].tu = 1.0f;
                pVertices3a[i+3].tv = 1.0f;
            }
        }
        else
        {
            CUSTOMVERTEX *pVertices3 = NULL;   
            pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices3, 0 );
            for(i=0; i<nNumVerts-1; i=i+2)
            {
                pVertices3[i].colour = D3DCOLOR_COLORVALUE( 0.0f , 1.0f, 1.0f, 1.0f);
                pVertices3[i+1].colour = D3DCOLOR_COLORVALUE( 0.0f , 0.5f, 0.0f, 1.0f);
            }
        }
        pTempVertexBuffer->Unlock();
        pTempVertexBuffer->Release();
    }
    else
    {
        MessageBox(NULL, "GetVertexBuffer 3 Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n GetVertexBuffer 3 Failed\n\n");
        return E_FAIL;
    }

    tempMesh   = NULL;
    D3DXCreatePolygon(g_pd3dDevice, 25.0f, 10, &tempMesh, NULL);
    if (testToRun == 4 || testToRun > 5)
    {
         tempMesh->CloneMeshFVF( 0, D3DFVF_CUSTOMVERTEX2, g_pd3dDevice, &g_PolygonMesh );
    }
    else
    {
         tempMesh->CloneMeshFVF( 0, D3DFVF_CUSTOMVERTEX, g_pd3dDevice, &g_PolygonMesh );
    }
    tempMesh->Release();

    if( SUCCEEDED( g_PolygonMesh->GetVertexBuffer( &pTempVertexBuffer ) ) )
    {
        nNumVerts = g_PolygonMesh->GetNumVertices();
        if (testToRun == 4 || testToRun > 5)
        {
            CUSTOMVERTEX2 *pVertices4a = NULL;
            pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices4a, 0 );
            for(i=0; i<nNumVerts-3; i=i+4)
            {
                pVertices4a[i].tu   = 0.0f;
                pVertices4a[i].tv   = 0.0f;
                pVertices4a[i+1].tu = 1.0f;
                pVertices4a[i+1].tv = 0.0f;
                pVertices4a[i+2].tu = 0.0f;
                pVertices4a[i+2].tv = 1.0f;
                pVertices4a[i+3].tu = 1.0f;
                pVertices4a[i+3].tv = 1.0f;
            }
        }
        else
        {
            CUSTOMVERTEX *pVertices4 = NULL;
            pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices4, 0 );
            for(i=0; i<nNumVerts-1; i=i+2)
            {
                pVertices4[i].colour   = D3DCOLOR_COLORVALUE( 0.7f , 0.0f, 0.7f, 1.0f);
                pVertices4[i+1].colour = D3DCOLOR_COLORVALUE( 0.0f , 0.0f, 1.0f, 1.0f);
            }
        }
        pTempVertexBuffer->Unlock();
        pTempVertexBuffer->Release();
    }
    else
    {
        MessageBox(NULL, "GetVertexBuffer 3 Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n GetVertexBuffer 3 Failed\n\n");
        return E_FAIL;
    }

    tempMesh   = NULL;
    D3DXCreateCylinder(g_pd3dDevice, 2.0f, 2.0f, 4.0f, 20, 2, &tempMesh, NULL);
    if (testToRun == 4 || testToRun > 5)
    {
        tempMesh->CloneMeshFVF( 0, D3DFVF_CUSTOMVERTEX2, g_pd3dDevice, &g_CylinderMesh );
    }
    else
    {
        tempMesh->CloneMeshFVF( 0, D3DFVF_CUSTOMVERTEX, g_pd3dDevice, &g_CylinderMesh );
    }
    tempMesh->Release();

    if( SUCCEEDED( g_CylinderMesh->GetVertexBuffer( &pTempVertexBuffer ) ) )
    {
        nNumVerts = g_CylinderMesh->GetNumVertices();
        if (testToRun == 4 || testToRun > 5)
        {
            CUSTOMVERTEX2 *pVertices5a = NULL;
            pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices5a, 0 );
            for(i=0; i<nNumVerts-3; i=i+4)
            {
                pVertices5a[i].tu   = 0.0f;
                pVertices5a[i].tv   = 0.0f;
                pVertices5a[i+1].tu = 1.0f;
                pVertices5a[i+1].tv = 0.0f;
                pVertices5a[i+2].tu = 0.0f;
                pVertices5a[i+2].tv = 1.0f;
                pVertices5a[i+3].tu = 1.0f;
                pVertices5a[i+3].tv = 1.0f;
            }
        }
        else
        {
            CUSTOMVERTEX *pVertices5 = NULL;
            pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices5, 0 );
            for(i=0; i<nNumVerts-1; i=i+2)
            {
                pVertices5[i].colour = D3DCOLOR_COLORVALUE( 1.0f , 0.6f, 0.0f, 1.0f);
                pVertices5[i+1].colour = D3DCOLOR_COLORVALUE( 0.3f , 1.0f, 0.0f, 1.0f);
            }
        }   
        pTempVertexBuffer->Unlock();
        pTempVertexBuffer->Release();
    }
    else
    {
        MessageBox(NULL, "GetVertexBuffer 4 Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n GetVertexBuffer 4 Failed\n\n");
        return E_FAIL;
    }

    tempMesh   = NULL;
    D3DXCreateBox(g_pd3dDevice, 4.0f, 3.0f, 4.0f, &tempMesh, NULL);
    if (testToRun == 4 || testToRun > 5)
    {
        tempMesh->CloneMeshFVF( 0, D3DFVF_CUSTOMVERTEX2, g_pd3dDevice, &g_BoxMesh );
    }
    else
    {
        tempMesh->CloneMeshFVF( 0, D3DFVF_CUSTOMVERTEX, g_pd3dDevice, &g_BoxMesh );
    }
    tempMesh->Release();

    if( SUCCEEDED( g_BoxMesh->GetVertexBuffer( &pTempVertexBuffer ) ) )
    {
        nNumVerts = g_BoxMesh->GetNumVertices();
        if (testToRun == 4 || testToRun > 5)
        {
            CUSTOMVERTEX2 *pVertices6a = NULL;
            pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices6a, 0 );
            for(i=0; i<nNumVerts-3; i=i+4)
            {
                pVertices6a[i].tu   = 0.0f;
                pVertices6a[i].tv   = 0.0f;
                pVertices6a[i+1].tu = 1.0f;
                pVertices6a[i+1].tv = 0.0f;
                pVertices6a[i+2].tu = 0.0f;
                pVertices6a[i+2].tv = 1.0f;
                pVertices6a[i+3].tu = 1.0f;
                pVertices6a[i+3].tv = 1.0f;
            }
        }
        else
        {
            CUSTOMVERTEX *pVertices6 = NULL;   
            pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices6, 0 );
            for(i=0; i<nNumVerts-1; i=i+2)
            {
                pVertices6[i].colour = D3DCOLOR_COLORVALUE( 0.9f , 0.0f, 0.0f, 1.0f);
                pVertices6[i+1].colour = D3DCOLOR_COLORVALUE( 0.3f , 0.0f, 1.0f, 1.0f);
            }
        }
        pTempVertexBuffer->Unlock();
        pTempVertexBuffer->Release();
    }
    else
    {
        MessageBox(NULL, "GetVertexBuffer 5 Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n GetVertexBuffer 5 Failed\n\n");
        return E_FAIL;
    }

    HDC hdc = CreateCompatibleDC( NULL );
    DWORD dwSize = 10;
    INT nHeight = -MulDiv( dwSize, GetDeviceCaps(hdc, LOGPIXELSY), 72 );
    HFONT hFont;
    HFONT hFontOld;

    hFont = CreateFont(nHeight, 0, 0, 0, FW_BOLD, FALSE, FALSE, FALSE, DEFAULT_CHARSET, 
        OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, "Courier New");
    
    hFontOld = (HFONT)SelectObject(hdc, hFont); 

    tempMesh   = NULL;
    D3DXCreateText( g_pd3dDevice, hdc, " DirectX 9 ", 0.001f, 0.4f, &tempMesh, NULL, NULL);
    tempMesh->CloneMeshFVF( 0, D3DFVF_CUSTOMVERTEX, g_pd3dDevice, &g_TextMesh );
    tempMesh->Release();
    if( SUCCEEDED( g_TextMesh->GetVertexBuffer( &pTempVertexBuffer ) ) )
    {
        nNumVerts = g_TextMesh->GetNumVertices();
        CUSTOMVERTEX *pVertices6t = NULL;   
        pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices6t, 0 );
        for(i=0; i<nNumVerts; i++)
        {
            if (testToRun == 5)
            {
                pVertices6t[i].colour = D3DCOLOR_COLORVALUE( 0.0f , 1.0f, 0.0f, 1.0f);
            }
            else
            {
                pVertices6t[i].colour = D3DCOLOR_COLORVALUE( 1.0f , 0.0f, 0.0f, 1.0f);
            }
        }
    
        pTempVertexBuffer->Unlock();
        pTempVertexBuffer->Release();
    }
    else
    {
        MessageBox(NULL, "GetVertexBuffer 6 Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n GetVertexBuffer 6 Failed\n\n");
        return E_FAIL;
    }

    SelectObject(hdc, hFontOld);
    DeleteObject( hFont );
    DeleteDC( hdc );

    CUSTOMVERTEX2 vertices4[] =
    {
        { aspc, 1.0f, -5.0f,  0.0f, 0.0f },
        { aspc, 1.0f,  5.0f, 16.0f, 0.0f },
        { aspc,-1.0f, -5.0f,  0.0f,16.0f },
        { aspc,-1.0f,  5.0f, 16.0f,16.0f },
        
        {-aspc, 1.0f, -5.0f, 16.0f, 0.0f },
        {-aspc,-1.0f, -5.0f, 16.0f,16.0f },
        {-aspc, 1.0f,  5.0f,  0.0f, 0.0f },
        {-aspc,-1.0f,  5.0f,  0.0f,16.0f }
    };

    CUSTOMVERTEX2 vertices4b[] =
    {
        {-aspc, 1.0f,  5.0f,  0.0f, 0.0f },
        { aspc, 1.0f,  5.0f, 16.0f, 0.0f },
        {-aspc, 1.0f, -5.0f,  0.0f,16.0f },
        { aspc, 1.0f, -5.0f, 16.0f,16.0f },
        
        {-aspc,-1.0f,  5.0f,  0.0f,0.0f },
        {-aspc,-1.0f, -5.0f,  8.0f,0.0f },
        { aspc,-1.0f,  5.0f,  0.0f,8.0f },
        { aspc,-1.0f, -5.0f,  8.0f,8.0f },  
    };

    g_tunnel_count = 8;
    vertSize = sizeof(vertices4);
    if( FAILED( g_pd3dDevice->CreateVertexBuffer( g_tunnel_count * sizeof(CUSTOMVERTEX2),
                                                  D3DUSAGE_WRITEONLY,
                                                  D3DFVF_CUSTOMVERTEX2,
                                                  D3DPOOL_DEFAULT,
                                                  &g_pVB4a,
                                                  NULL ) ) )
    {
        MessageBox(NULL, "CreateVertexBuffer 2 Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n CreateVertexBuffer 2 Failed\n\n");
        return E_FAIL;
    }

    VOID* pVertices7;
    if( FAILED( g_pVB4a->Lock( 0, vertSize, (void**)&pVertices7, 0 ) ) )
    {
        MessageBox(NULL, "Lock 2 Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n Lock 2 Failed\n\n");
        return E_FAIL;
    }
    memcpy( pVertices7, vertices4, vertSize );
    g_pVB4a->Unlock();

    vertSize = sizeof(vertices4b);
    if( FAILED( g_pd3dDevice->CreateVertexBuffer( g_tunnel_count * sizeof(CUSTOMVERTEX2),
                                                  D3DUSAGE_WRITEONLY,
                                                  D3DFVF_CUSTOMVERTEX2,
                                                  D3DPOOL_DEFAULT,
                                                  &g_pVB4b,
                                                  NULL ) ) )
    {
        MessageBox(NULL, "CreateVertexBuffer 3 Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n CreateVertexBuffer 3 Failed\n\n");
        return E_FAIL;
    }

    VOID* pVertices7b;
    if( FAILED( g_pVB4b->Lock( 0, vertSize, (void**)&pVertices7b, 0 ) ) )
    {
        MessageBox(NULL, "Lock 3 Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n Lock 3 Failed\n\n");
        return E_FAIL;
    }
    memcpy( pVertices7b, vertices4b, vertSize );
    g_pVB4b->Unlock();

    if (testToRun > 3)
    {
        // Use D3DX to create a texture from file based images
        if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, imageFiles[4], &g_pTexture_0 ) ) )
        {
            StringCchPrintf(amsg, 60, " %s file not found", imageFiles[4]);
            MessageBox(NULL, amsg, "ERROR", MB_OK);
            fprintf(outfile, "\n %s\n", amsg);
            fprintf(outfile, "  Commands: %s\n", commandLine);
            return E_FAIL;
        }
        if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, imageFiles[0], &g_pTexture_1 ) ) )
        {
            StringCchPrintf(amsg, 60, " %s file not found", imageFiles[0]);
            MessageBox(NULL, amsg, "ERROR", MB_OK);
            fprintf(outfile, "\n %s\n", amsg);
            fprintf(outfile, "  Commands: %s\n", commandLine);
            return E_FAIL;
        }
        if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, imageFiles[2], &g_pTexture_2 ) ) )
        {
            StringCchPrintf(amsg, 60, " %s file not found", imageFiles[2]);
            MessageBox(NULL, amsg, "ERROR", MB_OK);
            fprintf(outfile, "\n %s\n", amsg);
            fprintf(outfile, "  Commands: %s\n", commandLine);
            return E_FAIL;
        }
        if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, imageFiles[3], &g_pTexture_3 ) ) )
        {
            StringCchPrintf(amsg, 60, " %s file not found", imageFiles[3]);
            MessageBox(NULL, amsg, "ERROR", MB_OK);
            fprintf(outfile, "\n %s\n", amsg);
            fprintf(outfile, "  Commands: %s\n", commandLine);
            return E_FAIL;
        }
        if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, imageFiles[1], &g_pTexture_4 ) ) )
        {
            StringCchPrintf(amsg, 60, " %s file not found", imageFiles[1]);
            MessageBox(NULL, amsg, "ERROR", MB_OK);
            fprintf(outfile, "\n %s\n", amsg);
            fprintf(outfile, "  Commands: %s\n", commandLine);
            return E_FAIL;
        }


        if (testToRun < 7)
        {
            HRESULT hr;
            LPD3DXBUFFER pBufferErrors = NULL;
            hr = D3DXCreateEffectFromFile( g_pd3dDevice, 
                                               "OneEffect.fx",
                                               NULL, 
                                               NULL, 
                                               0, 
                                               NULL, 
                                               &g_pEffect, 
                                               &pBufferErrors );
    
            if( FAILED(hr) )
            {
                fprintf(outfile, "\nD3DXCreateEffectFromFile 1 failed\n");
                MessageBox(NULL, "D3DXCreateEffectFromFile 1 failed", "ERROR", MB_OK);
                return E_FAIL;
            }    
        }
        else
        {
            HRESULT hr;
            LPD3DXBUFFER pBufferErrors = NULL;    
            hr = D3DXCreateEffectFromFile( g_pd3dDevice, 
                                               "OneEffect.fx",
                                               NULL, 
                                               NULL, 
                                               0, 
                                               NULL, 
                                               &g_pEffect2, 
                                               &pBufferErrors );
    
            if( FAILED(hr) )
            {
                fprintf(outfile, "\nD3DXCreateEffectFromFile 2 failed\n");
                MessageBox(NULL, "D3DXCreateEffectFromFile 2 failed", "ERROR", MB_OK);
                return E_FAIL;
            }
        }
    }

    // 3 Cubes different colours
    CUSTOMVERTEX vertices5[] =
    {
          //Front face
       {-1.0f,-1.0f, 10.0f,0xFF00FFFF},{-1.0f, 1.0f, 10.0f,0xFFFFFF00},{ 1.0f, 1.0f, 10.0f,0xFFFFFFFF},
       { 1.0f, 1.0f, 10.0f,0xFFFFFFFF},{ 1.0f,-1.0f, 10.0f,0xFFFFFF00},{-1.0f,-1.0f, 10.0f,0xFF00FFFF},

          //Back face
       { 1.0f,-1.0f,12.0f,0xFF00FFFF},{ 1.0f, 1.0f,12.0f,0xFF00FFFF},{-1.0f, 1.0f,12.0f,0xFFFFFFFF},
       {-1.0f, 1.0f,12.0f,0xFFFFFFFF},{-1.0f,-1.0f,12.0f,0xFF00FFFF},{ 1.0f,-1.0f,12.0f,0xFF00FFFF},

          //Top face
       {-1.0f, 1.0f, 10.0f,0xFF0000FF},{-1.0f, 1.0f, 12.0f,0xFF00FF00},{ 1.0f, 1.0f,12.0f,0xFF0000FF},
       { 1.0f, 1.0f,12.0f ,0xFF0000FF},{ 1.0f, 1.0f, 10.0f,0xFF00FF00},{-1.0f, 1.0f,10.0f,0xFF0000FF},
    
          //Bottom face
       { 1.0f,-1.0f, 10.0f,0xFF0000FF},{ 1.0f,-1.0f,12.0f,0xFF00FF00},{-1.0f,-1.0f,12.0f,0xFF0000FF},
       {-1.0f,-1.0f,12.0f,0xFF0000FF},{-1.0f,-1.0f, 10.0f,0xFF00FF00},{ 1.0f,-1.0f,10.0f,0xFF0000FF},
    
          //Left face
       {-1.0f,-1.0f,12.0f,0xFF00FF00},{-1.0f, 1.0f,12.0f,0xFF00FFFF},{-1.0f, 1.0f, 10.0f,0xFF00FF00},
       {-1.0f, 1.0f, 10.0f,0xFF00FF00},{-1.0f,-1.0f, 10.0f,0xFF00FFFF},{-1.0f,-1.0f,12.0f,0xFF00FF00},
    
          //Right face
       { 1.0f,-1.0f, 10.0f,0xFF00FF00},{ 1.0f, 1.0f, 10.0f,0xFF00FFFF},{ 1.0f, 1.0f, 12.0f,0xFF00FF00},
       { 1.0f, 1.0f, 12.0f,0xFF00FF00},{ 1.0f,-1.0f, 12.0f,0xFF00FFFF},{ 1.0f,-1.0f, 10.0f,0xFF00FF00},

          //Front face *** 2
       {-1.0f,-1.0f, 10.0f,0xFFFF0000},{-1.0f, 1.0f, 10.0f,0xFFFF00FF},{ 1.0f, 1.0f, 10.0f,0xFFFF0000},
       { 1.0f, 1.0f, 10.0f,0xFFFF0000},{ 1.0f,-1.0f, 10.0f,0xFFFF00FF},{-1.0f,-1.0f, 10.0f,0xFFFF0000},

          //Back face
       { 1.0f,-1.0f,12.0f,0xFFFF0000},{ 1.0f, 1.0f,12.0f,0xFFFF00FF},{-1.0f, 1.0f,12.0f,0xFFFF0000},
       {-1.0f, 1.0f,12.0f,0xFFFF0000},{-1.0f,-1.0f,12.0f,0xFFFF00FF},{ 1.0f,-1.0f,12.0f,0xFFFF0000},

          //Top face
       {-1.0f, 1.0f, 10.0f,0xFFFF00FF},{-1.0f, 1.0f, 12.0f,0xFFFF0000},{ 1.0f, 1.0f, 12.0f,0xFFFF00FF},
       { 1.0f, 1.0f, 12.0f,0xFFFF00FF},{ 1.0f, 1.0f, 10.0f,0xFFFF0000},{-1.0f, 1.0f, 10.0f,0xFFFF00FF},
    
          //Bottom face
       { 1.0f,-1.0f, 10.0f,0xFFFF0000},{ 1.0f,-1.0f, 12.0f,0xFFFF00FF},{-1.0f,-1.0f,12.0f,0xFFFF0000},
       {-1.0f,-1.0f, 12.0f,0xFFFF0000},{-1.0f,-1.0f, 10.0f,0xFFFF00FF},{ 1.0f,-1.0f,10.0f,0xFFFF0000},
    
          //Left face
       {-1.0f,-1.0f, 12.0f,0xFFFF00FF},{-1.0f, 1.0f, 12.0f,0xFFFF0000},{-1.0f, 1.0f, 10.0f,0xFFFF00FF},
       {-1.0f, 1.0f, 10.0f,0xFFFF00FF},{-1.0f,-1.0f, 10.0f,0xFFFF0000},{-1.0f,-1.0f, 12.0f,0xFFFF00FF},
    
          //Right face
       { 1.0f,-1.0f, 10.0f,0xFFFF0000},{ 1.0f, 1.0f, 10.0f,0xFFFF00FF},{ 1.0f, 1.0f, 12.0f,0xFFFF0000},
       { 1.0f, 1.0f, 12.0f,0xFFFF0000},{ 1.0f,-1.0f, 12.0f,0xFFFF00FF},{ 1.0f,-1.0f, 10.0f,0xFFFF0000},

          //Front face 
       {-1.0f,-1.0f, 10.0f,0xFFFF00FF},{-1.0f, 1.0f, 10.0f,0xFFFFFF00},{ 1.0f, 1.0f, 10.0f,0xFFFFFFFF},
       { 1.0f, 1.0f, 10.0f,0xFFFFFFFF},{ 1.0f,-1.0f, 10.0f,0xFFFFFF00},{-1.0f,-1.0f, 10.0f,0xFFFF00FF},

          //Back face
       { 1.0f,-1.0f,12.0f,0xFFFF00FF},{ 1.0f, 1.0f,12.0f,0xFFFF00FF},{-1.0f, 1.0f,12.0f,0xFFFFFFFF},
       {-1.0f, 1.0f,12.0f,0xFFFFFFFF},{-1.0f,-1.0f,12.0f,0xFFFF00FF},{ 1.0f,-1.0f,12.0f,0xFFFF00FF},

          //Top face
       {-1.0f, 1.0f, 10.0f,0xFFFF0000},{-1.0f, 1.0f,12.0f,0xFF00FF00},{ 1.0f, 1.0f,12.0f,0xFFFF0000},
       { 1.0f, 1.0f,12.0f,0xFFFF0000},{ 1.0f, 1.0f, 10.0f,0xFF00FF00},{-1.0f, 1.0f,10.0f,0xFFFF0000},
    
          //Bottom face
       { 1.0f,-1.0f, 10.0f,0xFFFF0000},{ 1.0f,-1.0f,12.0f,0xFF00FF00},{-1.0f,-1.0f,12.0f,0xFFFF0000},
       {-1.0f,-1.0f,12.0f,0xFFFF0000},{-1.0f,-1.0f, 10.0f,0xFF00FF00},{ 1.0f,-1.0f,10.0f,0xFFFF0000},
    
          //Left face
       {-1.0f,-1.0f,12.0f,0xFF00FF00},{-1.0f, 1.0f,12.0f,0xFF00FFFF},{-1.0f, 1.0f, 10.0f,0xFF00FF00},
       {-1.0f, 1.0f, 10.0f,0xFF00FF00},{-1.0f,-1.0f, 10.0f,0xFF00FFFF},{-1.0f,-1.0f,12.0f,0xFF00FF00},
    
          //Right face
       { 1.0f,-1.0f, 10.0f,0xFF00FF00},{ 1.0f, 1.0f, 10.0f,0xFF00FFFF},{ 1.0f, 1.0f, 12.0f,0xFF00FF00},
       { 1.0f, 1.0f, 12.0f,0xFF00FF00},{ 1.0f,-1.0f, 12.0f,0xFF00FFFF},{ 1.0f,-1.0f, 10.0f,0xFF00FF00},   
    };

    vertSize5 = sizeof(vertices5);
    if( FAILED( g_pd3dDevice->CreateVertexBuffer( vertSize5,
                                                  D3DUSAGE_WRITEONLY,
                                                  D3DFVF_CUSTOMVERTEX,
                                                  D3DPOOL_DEFAULT,
                                                  &g_pVB2,
                                                  NULL ) ) )
    {
        MessageBox(NULL, "CreateVertexBuffer 4 Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n CreateVertexBuffer 4 Failed\n\n");
        return E_FAIL;
    }
    VOID* pVertices8;

    if( FAILED( g_pVB2->Lock( 0, vertSize5, (void**)&pVertices8, 0 ) ) )
    {
        MessageBox(NULL, "Lock 4 Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n Lock 4 Failed\n\n");
        return E_FAIL;
    }
    memcpy( pVertices8, vertices5, vertSize5 );
    g_pVB2->Unlock();
    vertSize5 = vertSize5 / 3;

    // Cubes for texturing
    CUSTOMVERTEX2 vertices6[] =
    {
        {-1.0f, 1.0f, 10.0f,  0.0f,0.0f },
        { 1.0f, 1.0f, 10.0f,  1.0f,0.0f },
        {-1.0f,-1.0f, 10.0f,  0.0f,1.0f },
        { 1.0f,-1.0f, 10.0f,  1.0f,1.0f },
        
        {-1.0f, 1.0f, 12.0f,  1.0f,0.0f },
        {-1.0f,-1.0f, 12.0f,  1.0f,1.0f },
        { 1.0f, 1.0f, 12.0f,  0.0f,0.0f },
        { 1.0f,-1.0f, 12.0f,  0.0f,1.0f },
        
        {-1.0f, 1.0f, 12.0f,  0.0f,0.0f },
        { 1.0f, 1.0f, 12.0f,  1.0f,0.0f },
        {-1.0f, 1.0f, 10.0f,  0.0f,1.0f },
        { 1.0f, 1.0f, 10.0f,  1.0f,1.0f },
        
        {-1.0f,-1.0f, 12.0f,  0.0f,0.0f },
        {-1.0f,-1.0f, 10.0f,  1.0f,0.0f },
        { 1.0f,-1.0f, 12.0f,  0.0f,1.0f },
        { 1.0f,-1.0f, 10.0f,  1.0f,1.0f },

        { 1.0f, 1.0f, 10.0f,  0.0f,0.0f },
        { 1.0f, 1.0f, 12.0f,  1.0f,0.0f },
        { 1.0f,-1.0f, 10.0f,  0.0f,1.0f },
        { 1.0f,-1.0f, 12.0f,  1.0f,1.0f },
        
        {-1.0f, 1.0f, 10.0f,  1.0f,0.0f },
        {-1.0f,-1.0f, 10.0f,  1.0f,1.0f },
        {-1.0f, 1.0f, 12.0f,  0.0f,0.0f },
        {-1.0f,-1.0f, 12.0f,  0.0f,1.0f }
    };

    if( FAILED(g_pd3dDevice->CreateVertexBuffer( 24*sizeof(CUSTOMVERTEX2), D3DUSAGE_WRITEONLY, 
                                  D3DFVF_CUSTOMVERTEX2, D3DPOOL_DEFAULT, 
                                  &g_pVB3, NULL )))
    {
        MessageBox(NULL, "CreateVertexBuffer 5 Failed", "ERROR", MB_OK);
        fprintf(outfile, "\n CreateVertexBuffer 5 Failed\n\n");
        return E_FAIL;
    }                              
    void *pVertices9 = NULL;

    g_pVB3->Lock( 0, sizeof(vertices6), (void**)&pVertices9, 0 );
    memcpy( pVertices9, vertices6, sizeof(vertices6) );
    g_pVB3->Unlock();

    if (testToRun > 6)
    {
        if (testToRun == 7 && pixelShaderV2)
        {
           //set up Pixel Shader
            LPD3DXBUFFER pBufferErrors = NULL;
    
           HRESULT hr = D3DXCreateEffectFromFile( g_pd3dDevice, 
                                               "pixel.shd",
                                               NULL, 
                                               NULL, 
                                               0, 
                                               NULL, 
                                               &g_pEffect, 
                                               &pBufferErrors );
    
            if( FAILED(hr) )
            {
                fprintf(outfile, "\nD3DXCreateEffectFromFile 3 failed\n");
                MessageBox(NULL, "D3DXCreateEffectFromFile 3 failed", "ERROR", MB_OK);
                return E_FAIL;
            }
        }
        else if (testToRun == 8 && vertexShaderV2)
        {
           //set up Vertex Shader
            LPD3DXBUFFER pBufferErrors = NULL;
    
           HRESULT hr = D3DXCreateEffectFromFile( g_pd3dDevice, 
                                               "vertex.shd",
                                               NULL, 
                                               NULL, 
                                               0, 
                                               NULL, 
                                               &g_pEffect, 
                                               &pBufferErrors );
    
            if( FAILED(hr) )
            {
                fprintf(outfile, "\nD3DXCreateEffectFromFile 4 failed\n");
                MessageBox(NULL, "D3DXCreateEffectFromFile 4 failed", "ERROR", MB_OK);
                return E_FAIL;
            }
        }
        else
        {
                LPD3DXBUFFER pBufferErrors = NULL;
                HRESULT hr = D3DXCreateEffectFromFile( g_pd3dDevice, 
                                                   "OneEffect.fx",
                                                   NULL, 
                                                   NULL, 
                                                   0, 
                                                   NULL, 
                                                   &g_pEffect, 
                                                   &pBufferErrors );
        
                if( FAILED(hr) )
                {
                    fprintf(outfile, "\nD3DXCreateEffectFromFile 5 failed\n");
                    MessageBox(NULL, "D3DXCreateEffectFromFile 5 failed", "ERROR", MB_OK);
                    return E_FAIL;
                }
        }
    }
    return S_OK;
} // InitVB


HRESULT InitGeometry()
{
    LPD3DXBUFFER pD3DXMtrlBuffer;

    // Load the mesh from the specified file
    if( FAILED( D3DXLoadMeshFromX( "egg.x", D3DXMESH_SYSTEMMEM, 
                                   g_pd3dDevice, NULL, 
                                   &pD3DXMtrlBuffer, NULL, &g_dwNumMaterials, 
                                   &g_pMesh ) ) )
    {
        MessageBox(NULL, "Could not find egg.x", "ERROR", MB_OK);
        fprintf(outfile, "\n Could not find egg.x\n\n");
        return E_FAIL;
    }

    D3DXMATERIAL* d3dxMaterials = (D3DXMATERIAL*)pD3DXMtrlBuffer->GetBufferPointer();
    g_pMeshMaterials = new D3DMATERIAL9[g_dwNumMaterials];
    if( g_pMeshMaterials == NULL )
    {
        MessageBox(NULL, "D3DXMATERIAL E_OUTOFMEMORY", "ERROR", MB_OK);
        fprintf(outfile, "\n D3DXMATERIAL E_OUTOFMEMORY\n\n");
        return E_OUTOFMEMORY;
    }
    g_pMeshTextures  = new LPDIRECT3DTEXTURE9[g_dwNumMaterials];
    if( g_pMeshTextures == NULL )
    {
        MessageBox(NULL, "LPDIRECT3DTEXTURE9 E_OUTOFMEMORY", "ERROR", MB_OK);
        fprintf(outfile, "\n LPDIRECT3DTEXTURE9 E_OUTOFMEMORY\n\n");
        return E_OUTOFMEMORY;
    }
    for( DWORD i=0; i<g_dwNumMaterials; i++ )
    {
        g_pMeshMaterials[i] = d3dxMaterials[i].MatD3D;
        g_pMeshMaterials[i].Ambient = g_pMeshMaterials[i].Diffuse;
        g_pMeshTextures[i] = NULL;

        if( d3dxMaterials[i].pTextureFilename != NULL && 
            lstrlen(d3dxMaterials[i].pTextureFilename) > 0 )
        {
            // Create the texture
            if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, 
                                                d3dxMaterials[i].pTextureFilename, 
                                                &g_pMeshTextures[i] ) ) )
            {
                 fprintf(outfile, "\n Could not find texture map\n\n");
                 MessageBox(NULL, "Could not find texture map", "ERROR", MB_OK);
            }
        }        
    }
    pD3DXMtrlBuffer->Release();

    return S_OK;
} // InitGeometry


VOID Cleanup()
{
    if( g_pTexture_0 != NULL )
        g_pTexture_0->Release();

    if( g_pTexture_1 != NULL )
        g_pTexture_1->Release();

    if( g_pTexture_2 != NULL )
        g_pTexture_2->Release();

    if( g_pTexture_3 != NULL )
        g_pTexture_3->Release();

    if( g_pTexture_4 != NULL )
        g_pTexture_4->Release();
 
    if( g_pEffect2 != NULL )
        g_pEffect2->Release();

    if( g_pEffect != NULL )
        g_pEffect->Release();

    if( g_pVB4b != NULL )        
        g_pVB4b->Release();

    if( g_pVB4a != NULL )        
        g_pVB4a->Release();

    if( g_pVB3 != NULL )        
        g_pVB3->Release();

    if( g_pVB2 != NULL )        
        g_pVB2->Release();

    if( g_pVB1 != NULL )        
        g_pVB1->Release();

    if( g_pMeshMaterials != NULL ) 
        delete[] g_pMeshMaterials;

    if( g_pMeshTextures )
    {
        for( DWORD i = 0; i < g_dwNumMaterials; i++ )
        {
            if( g_pMeshTextures[i] )
                g_pMeshTextures[i]->Release();
        }
        delete[] g_pMeshTextures;
    }
    if( g_pMesh != NULL )
        g_pMesh->Release();

    if( g_TextMesh != NULL )
        g_TextMesh->Release();

    if( g_BoxMesh != NULL )
        g_BoxMesh->Release();

    if( g_CylinderMesh != NULL )
        g_CylinderMesh->Release();

    if( g_PolygonMesh != NULL )
        g_PolygonMesh->Release();

    if( g_TorusMesh != NULL )
        g_TorusMesh->Release();

    if( g_TeaPotMesh != NULL )
        g_TeaPotMesh->Release();

    if( g_SphereMesh != NULL )
        g_SphereMesh->Release();

    if( g_pd3dDevice != NULL )
        g_pd3dDevice->Release();

    if( g_pD3D != NULL )
        g_pD3D->Release();
} // Cleanup


VOID SetupMatrices()
{
    D3DXMATRIXA16 matWorld;
    D3DXMATRIXA16 matView;
    D3DXMATRIXA16 matProj;
    D3DXVECTOR3 vLookatPt2;
    D3DXVECTOR3 vUpVec2;
    D3DXMATRIX world_matrix;
       
    if (testToRun < 4)
    {
        D3DXMatrixRotationY( &matWorld, timeGetTime()/1000.0f );
        g_pd3dDevice->SetTransform( D3DTS_WORLD, &matWorld );
        D3DXVECTOR3 vEyePt( 0.0f, 3.0f,-7.0f );  // smaller
        D3DXVECTOR3 vLookatPt( 0.0f, 0.0f, 0.0f );
        D3DXVECTOR3 vUpVec( 1.0f, 1.0f, 0.0f );
        D3DXMatrixLookAtLH( &matView, &vEyePt, &vLookatPt, &vUpVec );
        g_pd3dDevice->SetTransform( D3DTS_VIEW, &matView );
        D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI/4, aspect, 1.0f, 100.0f );
        g_pd3dDevice->SetTransform( D3DTS_PROJECTION, &matProj );
    }
    else
    {     
        vEyePt2.x = 0.0f;
        vEyePt2.y = 0.0f;
        vLookatPt2.x  = 0.0f;
        vLookatPt2.y  = 0.0f;
        vLookatPt2.z  = 10.0f;
        vUpVec2.x = 0.0f;
        vUpVec2.y = 1.0f;
        vUpVec2.z = 0.0f;
        D3DXMatrixLookAtLH( &matView2, &vEyePt2, &vLookatPt2, &vUpVec2 );
        g_pd3dDevice->SetTransform( D3DTS_VIEW, &matView2 );
        D3DXMatrixPerspectiveFovLH( &matProj2, D3DX_PI/4, aspect, 1.0f, 100.0f );
        g_pd3dDevice->SetTransform( D3DTS_PROJECTION, &matProj2 );

        // For independent movement of objects
        D3DXMatrixIdentity(&world_matrix);
        g_pd3dDevice->SetTransform(D3DTS_WORLD,&world_matrix);

        if (testToRun == 4)
        {
            vEyePt2.z = vEyePt2.z + vEyeInc;
            if (vEyePt2.z >  vEyeIn || vEyePt2.z < vEyeOut)
            {
                vEyeInc = - vEyeInc;
            }
        }
        else
        {
           vEyePt2.z = -7.0f;
        } 
    }    
} // SetupMatrices

//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Draws the scene
//-----------------------------------------------------------------------------
VOID Render()
{
    int i;
    float incv;
    
    // Clear the backbuffer and the zbuffer
    g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, 
                         D3DCOLOR_XRGB(0,0,0), 1.0f, 0 );
    
    // Begin the scene
    if( SUCCEEDED( g_pd3dDevice->BeginScene() ) )
    {
        // Setup the world, view, and projection matrices
        SetupMatrices();

        if (testToRun < 4)
        {
            for( DWORD i=0; i<g_dwNumMaterials; i++ )
            {
                // Set the material and texture for this subset
                g_pd3dDevice->SetMaterial( &g_pMeshMaterials[i] );
                g_pd3dDevice->SetTexture( 0, g_pMeshTextures[i] );
            
                // Draw the mesh subset
                g_pMesh->DrawSubset( i );
            }
        }
        if (testToRun == 3)
        {
            // 500 cubes
            g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX );        
            g_pd3dDevice->SetStreamSource( 0, g_pVB1, 0, sizeof(CUSTOMVERTEX) );
            g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, g_cube_count );
        }
        else if (testToRun == 4)
        {
            // Tunnel
            g_pEffect->SetTechnique( "OneTexture" );
            g_pEffect->SetTexture("texture0", g_pTexture_4);
        
            UINT uPasses;
            g_pEffect->Begin( &uPasses, 0 );
            g_pEffect->BeginPass(0);
            g_pd3dDevice->SetStreamSource( 0, g_pVB4a, 0, sizeof(CUSTOMVERTEX2) );
            g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX2 );
            for (i=0; i<g_tunnel_count-3; i=i+4)
            {        
                g_pd3dDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP, i, 2);
            }
            g_pEffect->EndPass();
            g_pEffect->End();   

            g_pEffect->SetTechnique( "OneTexture" );
            g_pEffect->SetTexture("texture0", g_pTexture_3);
        
            UINT uPasses2;
            g_pEffect->Begin( &uPasses2, 0 );
            g_pEffect->BeginPass(0);
            g_pd3dDevice->SetStreamSource( 0, g_pVB4b, 0, sizeof(CUSTOMVERTEX2) );
            g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX2 );
            for (i=0; i<g_tunnel_count-3; i=i+4)
            {        
                g_pd3dDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP, i, 2);
            }
            g_pEffect->EndPass();
            g_pEffect->End();   

            rotPos = 0;
            rotateInc = 0.02f * rotateAdj;
            renderSphere( 0.0f, 2.0f, 20.0f);
            rotPos = 1;
            rotateInc = 0.03f * rotateAdj;
            renderCylinder( -2.0f, -2.0f, 15.0f);
            rotPos = 2;
            rotateInc = 0.025f * rotateAdj;
            renderTorus( 3.0f, -2.0f, 25.0f);

            rotPos = 6;
            rotateInc = 0.01f * rotateAdj;
            renderCube3( -4.0f,  2.0f,  14.0f);
            rotPos = 7;
            rotateInc = 0.006f * rotateAdj;
            renderCube4( -3.0f,  2.5f,  16.0f);
            rotPos = 8;
            rotateInc = 0.012f * rotateAdj;
            renderCube1( -7.0f,  2.0f, 28.0f);
            rotPos = 9;
            rotateInc = 0.008f * rotateAdj;
            renderCube2( -6.0f,  2.5f, 30.0f);

        }
        else if (testToRun > 4)
        {
            if (testToRun > 6) 
            {
                incv = (float)(0.5 / fps);
                if (incv > 0.5f) incv = 0.5f;
                if (flip == 0)
                {
                    pixVary = pixVary - incv;
                    if (pixVary < 0.01f)
                    {
                        flip = 1;
                        pixVary = 0.01f;
                    }
                }
                else
                {
                    pixVary = pixVary + incv;
                    if (pixVary > 1.0)
                    {
                        flip = 0;
                        pixVary = 1.0f;
                    }
                }
                pv[3] = 1.0f;
                g_pEffect2->SetTechnique( "OneTexture" );
            }
            else
            {
                g_pEffect->SetTechnique( "OneTexture" );
            }
            if (testToRun > 6)
            {
                pv[0] = 1.0f - pixVary;
                pv[1] = pixVary;
                pv[2] = 1.0f - pixVary;
                g_pEffect->SetTechnique( "Pshade" );
                g_pEffect->SetFloatArray("f4Vary", pv, 4);
                g_pEffect->SetFloat("fVary", pixVary);
            }
            rotPos = 0;
            rotateInc = 0.003f * rotateAdj;
            renderCube1( -2.0f, 1.0f, 40.0f);
            rotPos = 1;
            rotateInc = 0.002f * rotateAdj;
            renderCube1(  2.0f,-1.5f, 30.0f);
            rotPos = 2;
            rotateInc = 0.001f * rotateAdj;
            renderCube1(  3.0f,-2.0f, 10.0f);
      
            rotPos = 3;
            rotateInc = 0.004f * rotateAdj;
            renderCube2(  3.0f,-1.0f, 35.0f);
            rotPos = 4;
            rotateInc = 0.002f * rotateAdj;
            renderCube2( -3.0f, 1.7f, 25.0f);
            rotPos = 5;
            rotateInc = 0.001f * rotateAdj;
            renderCube2( -2.3f,-1.7f, 12.0f);

            if (testToRun > 6)
            {
                pv[0] = pixVary;
                pv[1] = 1.0f - pixVary;
                pv[2] = 1.0f - pixVary;
                g_pEffect->SetFloatArray("f4Vary", pv, 4);
            }
            rotPos = 6;
            rotateInc = 0.005f * rotateAdj;
            renderCube3( -9.0f,  1.0f,  4.0f);
            rotPos = 7;
            rotateInc = 0.003f * rotateAdj;
            renderCube4( -8.0f,  1.5f,  6.0f);
            rotPos = 8;
            rotateInc = 0.006f * rotateAdj;
            renderCube3( -7.0f,  2.0f, 8.0f);
            rotPos = 9;
            rotateInc = 0.004f * rotateAdj;
            renderCube4( -6.0f,  2.5f, 10.0f);
            rotPos = 10;
            rotateInc = 0.007f * rotateAdj;
            renderCube3( -5.0f,  3.0f,  14.0f);
            rotPos = 11;
            rotateInc = 0.006f * rotateAdj;
            renderCube4( -4.0f,  3.5f,  16.0f);
            rotPos = 12;
            rotateInc = 0.008f * rotateAdj;
            renderCube3( -3.0f,  4.0f, 18.0f);
            rotPos = 13;
            rotateInc = 0.002f * rotateAdj;
            renderCube4( -2.0f,  4.5f, 20.0f);
             
            if (testToRun > 6)
            {
                pv[0] = 1.0f - pixVary;
                pv[1] = 1.0f - pixVary;
                pv[2] = pixVary;
                g_pEffect->SetFloatArray("f4Vary", pv, 4);
            }
            rotPos = 14;
            rotateInc = 0.003f * rotateAdj;
            renderSphere(  1.3f,-0.5f, 5.0f);
            rotPos = 15;
            rotateInc = 0.002f * rotateAdj;
            renderSphere( -2.5f, 7.0f, 15.0f);

            if (testToRun > 6)
            {
                pv[0] = pixVary;
                pv[1] = 1.0f - pixVary;
                pv[2] = 1.0f - pixVary;
                g_pEffect->SetFloatArray("f4Vary", pv, 4);
            }
            rotPos = 16;
            rotateInc = 0.004f * rotateAdj;
            renderTeaPot(  3.0f, 2.0f, 2.0f);
            rotPos = 17;
            rotateInc = 0.002f * rotateAdj;
            renderTeaPot( -4.0f,-7.0f, 15.0f);

            if (testToRun > 6)
            {
                pv[0] = 1.0f - pixVary;
                pv[1] = pixVary;
                pv[2] = 1.0f - pixVary;
                g_pEffect->SetFloatArray("f4Vary", pv, 4);
            }
            rotPos = 18;
            rotateInc = 0.006f * rotateAdj;
            renderTorus(   7.0f,-8.0f, 20.0f);
            rotPos = 19;
            rotateInc = 0.002f * rotateAdj;
            renderTorus(  -10.0f,-1.7f, 40.0f);

            rotPos = 20;
            rotateInc = 0.003f * rotateAdj;
            renderPolygon( 0.0f, 0.0f, 60.0f);

            if (testToRun > 6)
            {
                pv[0] = pixVary;
                pv[1] = 1.0f - pixVary;
                pv[2] = 1.0f - pixVary;
                g_pEffect->SetFloatArray("f4Vary", pv, 4);
            }
            rotPos = 21;
            rotateInc = 0.008f * rotateAdj;
            renderCylinder(-6.0f, 5.0f, 12.0f);
            rotPos = 22;
            rotateInc = 0.003f * rotateAdj;
            renderCylinder(-4.0f,-4.5f, 25.0f);

            rotPos = 23;
            rotateInc = 0.004f * rotateAdj;
            renderBox(-6.5f, -4.0f, 12.0f);
            rotPos = 24;
            rotateInc = 0.003f * rotateAdj;
            renderBox( 1.5f, 6.0f, 22.0f);

            rotPos = 25;
            rotateInc = 0.002f * rotateAdj;
            renderText( 0.5f, 2.0f, 10.0f);
        }
    
        // End the scene
        g_pd3dDevice->EndScene();
    }
    // Present the backbuffer contents to the display
    g_pd3dDevice->Present( NULL, NULL, NULL, NULL );
} // Render

void renderText(float x, float y, float z)
{
   D3DXMATRIX rotate_matrix;
   D3DXMATRIX trans_matrix;
   D3DXMATRIX world_matrix;

   D3DXMatrixRotationZ(&rotate_matrix, rotateCount[rotPos]);
   D3DXMatrixTranslation(&trans_matrix, x, y, z);
   D3DXMatrixMultiply(&world_matrix, &rotate_matrix, &trans_matrix);
   g_pd3dDevice->SetTransform(D3DTS_WORLD,&world_matrix);

   g_TextMesh->DrawSubset(0);

   rotateCount[rotPos] = rotateCount[rotPos] - rotateInc;
   if(rotateCount[rotPos] < 0)
   {  
      rotateCount[rotPos] =  D3DX_PI*2;
   }
}

void renderBox(float x, float y, float z)
{
   D3DXMATRIX rotate_matrix;
   D3DXMATRIX trans_matrix;
   D3DXMATRIX world_matrix;

   D3DXMatrixRotationZ(&rotate_matrix, rotateCount[rotPos]);
   D3DXMatrixTranslation(&trans_matrix, x, y, z);
   D3DXMatrixMultiply(&world_matrix, &rotate_matrix, &trans_matrix);
   g_pd3dDevice->SetTransform(D3DTS_WORLD,&world_matrix);
   D3DXMATRIXA16 mWorldViewProjection;
   mWorldViewProjection = world_matrix * matView2 * matProj2;

    if (testToRun == 4 || testToRun > 5)
    {
        g_pEffect->SetTexture("texture0", g_pTexture_2);
        if (testToRun == 8)
        {
            g_pEffect->SetMatrix("WorldViewProj", &mWorldViewProjection );
        }
    
        UINT uPasses;
        g_pEffect->Begin( &uPasses, 0 );
        g_pEffect->BeginPass(0);
        g_BoxMesh->DrawSubset(0);    
        g_pEffect->EndPass();
        g_pEffect->End();
    }
    else
    { 
        g_BoxMesh->DrawSubset(0);    
    }
   rotateCount[rotPos] = rotateCount[rotPos] - rotateInc;
   if(rotateCount[rotPos] < 0)
   {  
      rotateCount[rotPos] =  D3DX_PI*2;
   }
}

void renderCylinder(float x, float y, float z)
{
    D3DXMATRIX rotate_matrix;
    D3DXMATRIX trans_matrix;
    D3DXMATRIX world_matrix;

   D3DXMatrixRotationZ(&rotate_matrix, rotateCount[rotPos]);
   D3DXMatrixTranslation(&trans_matrix, x, y, z);
   D3DXMatrixMultiply(&world_matrix, &rotate_matrix, &trans_matrix);
   g_pd3dDevice->SetTransform(D3DTS_WORLD,&world_matrix);
   D3DXMATRIXA16 mWorldViewProjection;
   mWorldViewProjection = world_matrix * matView2 * matProj2;

    if (testToRun  == 4 || testToRun > 5)
    {
        g_pEffect->SetTexture("texture0", g_pTexture_4);
        if (testToRun == 8)
        {
            g_pEffect->SetMatrix("WorldViewProj", &mWorldViewProjection );
        }
    
        UINT uPasses;
        g_pEffect->Begin( &uPasses, 0 );
        g_pEffect->BeginPass(0);
        g_CylinderMesh->DrawSubset(0);
        g_pEffect->EndPass();
        g_pEffect->End();
    }
    else
    { 
        g_CylinderMesh->DrawSubset(0);
    }
   rotateCount[rotPos] = rotateCount[rotPos] + rotateInc;
   if(rotateCount[rotPos] > D3DX_PI*2)
   {  
      rotateCount[rotPos] = 0.0f;
   }
}

void renderPolygon(float x, float y, float z)
{
    D3DXMATRIX rotate_matrix;
    D3DXMATRIX trans_matrix;
    D3DXMATRIX world_matrix;

   D3DXMatrixRotationZ(&rotate_matrix, rotateCount[rotPos]);
   D3DXMatrixTranslation(&trans_matrix, x, y, z);
   D3DXMatrixMultiply(&world_matrix, &rotate_matrix, &trans_matrix);
   g_pd3dDevice->SetTransform(D3DTS_WORLD,&world_matrix);

    if (testToRun  == 4 || testToRun > 5)
    {

        if (testToRun  >  6)
        {
            g_pEffect2->SetTexture("texture0", g_pTexture_1);
        
            UINT uPasses;
            g_pEffect2->Begin( &uPasses, 0 );
            g_pEffect2->BeginPass(0);
            g_PolygonMesh->DrawSubset(0);    
            g_pEffect2->EndPass();
            g_pEffect2->End();
        }
        else
        {
            g_pEffect->SetTexture("texture0", g_pTexture_1);
        
            UINT uPasses;
            g_pEffect->Begin( &uPasses, 0 );
            g_pEffect->BeginPass(0);
            g_PolygonMesh->DrawSubset(0);    
            g_pEffect->EndPass();
            g_pEffect->End();
        }
    }
    else
    { 
        g_PolygonMesh->DrawSubset(0);    
    }
   rotateCount[rotPos] = rotateCount[rotPos] - rotateInc;
   if(rotateCount[rotPos] < 0)
   {  
      rotateCount[rotPos] = D3DX_PI*2;
   }
}

void renderTorus(float x, float y, float z)
{
    D3DXMATRIX rotate_matrix;
    D3DXMATRIX trans_matrix;
    D3DXMATRIX world_matrix;

   D3DXMatrixRotationX(&rotate_matrix, rotateCount[rotPos]);
   D3DXMatrixTranslation(&trans_matrix, x, y, z);
   D3DXMatrixMultiply(&world_matrix, &rotate_matrix, &trans_matrix);
   g_pd3dDevice->SetTransform(D3DTS_WORLD,&world_matrix);
   D3DXMATRIXA16 mWorldViewProjection;
   mWorldViewProjection = world_matrix * matView2 * matProj2;

    if (testToRun  == 4 || testToRun > 5)
    {
        g_pEffect->SetTexture("texture0", g_pTexture_4);
        if (testToRun == 8)
        {
            g_pEffect->SetMatrix("WorldViewProj", &mWorldViewProjection );
        }
    
        UINT uPasses;
        g_pEffect->Begin( &uPasses, 0 );
        g_pEffect->BeginPass(0);
        g_TorusMesh->DrawSubset(0);    
        g_pEffect->EndPass();
        g_pEffect->End();
    }
    else
    { 
        g_TorusMesh->DrawSubset(0);    
    }
   rotateCount[rotPos] = rotateCount[rotPos] - rotateInc;
   if(rotateCount[rotPos] < 0)
   {  
      rotateCount[rotPos] = D3DX_PI*2;
   }
}

void renderTeaPot(float x, float y, float z)
{
    D3DXMATRIX rotate_matrix;
    D3DXMATRIX trans_matrix;
    D3DXMATRIX world_matrix;

   D3DXMatrixRotationY(&rotate_matrix, rotateCount[rotPos]);
   D3DXMatrixTranslation(&trans_matrix, x, y, z);
   D3DXMatrixMultiply(&world_matrix, &rotate_matrix, &trans_matrix);
   g_pd3dDevice->SetTransform(D3DTS_WORLD,&world_matrix);
   D3DXMATRIXA16 mWorldViewProjection;
   mWorldViewProjection = world_matrix * matView2 * matProj2;

    if (testToRun  == 4 || testToRun > 5)
    {
        g_pEffect->SetTexture("texture0", g_pTexture_2);
        if (testToRun == 8)
        {
            g_pEffect->SetMatrix("WorldViewProj", &mWorldViewProjection );
        }
    
        UINT uPasses;
        g_pEffect->Begin( &uPasses, 0 );
        g_pEffect->BeginPass(0);
        g_TeaPotMesh->DrawSubset(0);
        g_pEffect->EndPass();
        g_pEffect->End();
    }
    else
    { 
        g_TeaPotMesh->DrawSubset(0);
    }   

   rotateCount[rotPos] = rotateCount[rotPos] + rotateInc;
   if(rotateCount[rotPos] < 0)
   {  
      rotateCount[rotPos] = D3DX_PI*2;
   }
}

void renderSphere(float x, float y, float z)
{
    D3DXMATRIX rotate_matrix;
    D3DXMATRIX trans_matrix;
    D3DXMATRIX world_matrix;

   D3DXMatrixRotationX(&rotate_matrix, rotateCount[rotPos]);
   D3DXMatrixTranslation(&trans_matrix, x, y, z);
   D3DXMatrixMultiply(&world_matrix, &rotate_matrix, &trans_matrix);
   g_pd3dDevice->SetTransform(D3DTS_WORLD,&world_matrix);
   D3DXMATRIXA16 mWorldViewProjection;
   mWorldViewProjection = world_matrix * matView2 * matProj2;

    if (testToRun  == 4 || testToRun > 5)
    {
        g_pEffect->SetTexture("texture0", g_pTexture_0);
        if (testToRun == 8)
        {
            g_pEffect->SetMatrix("WorldViewProj", &mWorldViewProjection );
        }
    
        UINT uPasses;
        g_pEffect->Begin( &uPasses, 0 );
        g_pEffect->BeginPass(0);
        g_SphereMesh->DrawSubset(0);    
        g_pEffect->EndPass();
        g_pEffect->End();
    }
    else
    { 
       g_SphereMesh->DrawSubset(0);    
    }
   rotateCount[rotPos] = rotateCount[rotPos] + rotateInc;
   if(rotateCount[rotPos] > D3DX_PI*2)
   {  
      rotateCount[rotPos] = 0.0f;
   }
}


void renderCube1(float x, float y, float z)
{
    int i;

    D3DXMATRIX rotate_matrix;
    D3DXMATRIX trans_matrix;
    D3DXMATRIX world_matrix;

   D3DXMatrixRotationX(&rotate_matrix, rotateCount[rotPos]);
   D3DXMatrixTranslation(&trans_matrix, x, y, z);
   D3DXMatrixMultiply(&world_matrix, &rotate_matrix, &trans_matrix);
   g_pd3dDevice->SetTransform(D3DTS_WORLD,&world_matrix);
   D3DXMATRIXA16 mWorldViewProjection;
   mWorldViewProjection = world_matrix * matView2 * matProj2;

    if (testToRun  == 4 || testToRun > 5)
    {
        g_pEffect->SetTexture("texture0", g_pTexture_2);
        if (testToRun == 8)
        {
            g_pEffect->SetMatrix("WorldViewProj", &mWorldViewProjection );
        }
    
        UINT uPasses;
        g_pEffect->Begin( &uPasses, 0 );
        g_pEffect->BeginPass(0);
        g_pd3dDevice->SetStreamSource( 0, g_pVB3, 0, sizeof(CUSTOMVERTEX2) );
        g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX2 );
        for (i=0; i<21; i=i+4)
        {        
           g_pd3dDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,  i, 2);
        }
        g_pEffect->EndPass();
        g_pEffect->End();
    }
    else
    {
       g_pd3dDevice->SetStreamSource( 0, g_pVB2, 0, sizeof(CUSTOMVERTEX) );
       g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX );        
       g_pd3dDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 0, g_cube_count2);
    }
   rotateCount[rotPos] = rotateCount[rotPos] + rotateInc;
   if(rotateCount[rotPos] > D3DX_PI*2)
   {  
      rotateCount[rotPos] = 0.0f;
   }
}

void renderCube2(float x, float y, float z)
{
   D3DXMATRIX rotate_matrix;
   D3DXMATRIX trans_matrix;
   D3DXMATRIX world_matrix;
   int i;

   D3DXMatrixRotationY(&rotate_matrix, rotateCount[rotPos]);
   D3DXMatrixTranslation(&trans_matrix, x, y, z);
   D3DXMatrixMultiply(&world_matrix, &rotate_matrix, &trans_matrix);
   g_pd3dDevice->SetTransform(D3DTS_WORLD,&world_matrix);
   D3DXMATRIXA16 mWorldViewProjection;
   mWorldViewProjection = world_matrix * matView2 * matProj2;

    if (testToRun  == 4 || testToRun > 5)
    {
        g_pEffect->SetTexture("texture0", g_pTexture_2);
        if (testToRun == 8)
        {
            g_pEffect->SetMatrix("WorldViewProj", &mWorldViewProjection );
        }
    
        UINT uPasses;
        g_pEffect->Begin( &uPasses, 0 );
        g_pEffect->BeginPass(0);
        g_pd3dDevice->SetStreamSource( 0, g_pVB3, 0, sizeof(CUSTOMVERTEX2) );
        g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX2 );
        for (i=0; i<21; i=i+4)
        {        
           g_pd3dDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,  i, 2);
        }
        g_pEffect->EndPass();
        g_pEffect->End();
    }
    else
    {
       g_pd3dDevice->SetStreamSource( 0, g_pVB2, vertSize5, sizeof(CUSTOMVERTEX) );
       g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX );
       g_pd3dDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 0, g_cube_count2);
    }
   rotateCount[rotPos] = rotateCount[rotPos] + rotateInc;
   if(rotateCount[rotPos] > D3DX_PI*2)
   {  
      rotateCount[rotPos] = 0.0f;
   }
}

void renderCube3(float x, float y, float z)
{    
   D3DXMATRIX rotate_matrix;
   D3DXMATRIX trans_matrix;
   D3DXMATRIX world_matrix;
   int i;

   D3DXMatrixRotationZ(&rotate_matrix, rotateCount[rotPos]);
   D3DXMatrixTranslation(&trans_matrix, x, y, z);
   D3DXMatrixMultiply(&world_matrix, &rotate_matrix, &trans_matrix);
   g_pd3dDevice->SetTransform(D3DTS_WORLD,&world_matrix);
   D3DXMATRIXA16 mWorldViewProjection;
   mWorldViewProjection = world_matrix * matView2 * matProj2;

    if (testToRun  == 4 || testToRun > 5)
    {
        g_pEffect->SetTexture("texture0", g_pTexture_0);
        if (testToRun == 8)
        {
            g_pEffect->SetMatrix("WorldViewProj", &mWorldViewProjection );
        }
    
        UINT uPasses;
        g_pEffect->Begin( &uPasses, 0 );
        g_pEffect->BeginPass(0);
        g_pd3dDevice->SetStreamSource( 0, g_pVB3, 0, sizeof(CUSTOMVERTEX2) );
        g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX2 );
        for (i=0; i<21; i=i+4)
        {        
           g_pd3dDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,  i, 2);
        }
        g_pEffect->EndPass();
        g_pEffect->End();
    }
    else
    {
         g_pd3dDevice->SetStreamSource( 0, g_pVB2, vertSize5 * 2, sizeof(CUSTOMVERTEX) );
         g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX );        
         g_pd3dDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 0, g_cube_count2);
    }

   rotateCount[rotPos] = rotateCount[rotPos] + rotateInc;
   if(rotateCount[rotPos] > D3DX_PI*2)
   {  
      rotateCount[rotPos] = 0.0f;
   }
}

void renderCube4(float x, float y, float z)
{
    D3DXMATRIX rotate_matrix;
    D3DXMATRIX trans_matrix;
    D3DXMATRIX world_matrix;
    int i;
    
   D3DXMatrixRotationZ(&rotate_matrix, rotateCount[rotPos]);
   D3DXMatrixTranslation(&trans_matrix, x, y, z);
   D3DXMatrixMultiply(&world_matrix, &rotate_matrix, &trans_matrix);
   g_pd3dDevice->SetTransform(D3DTS_WORLD,&world_matrix);
   D3DXMATRIXA16 mWorldViewProjection;
   mWorldViewProjection = world_matrix * matView2 * matProj2;

    if (testToRun  == 4 || testToRun > 5)
    {
        g_pEffect->SetTexture("texture0", g_pTexture_0);
        if (testToRun == 8)
        {
            g_pEffect->SetMatrix("WorldViewProj", &mWorldViewProjection );
        }
    
        UINT uPasses;
        g_pEffect->Begin( &uPasses, 0 );
        g_pEffect->BeginPass(0);
        g_pd3dDevice->SetStreamSource( 0, g_pVB3, 0, sizeof(CUSTOMVERTEX2) );
        g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX2 );
        for (i=0; i<21; i=i+4)
        {        
           g_pd3dDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,  i, 2);
        }
        g_pEffect->EndPass();
        g_pEffect->End();
    }
    else
    {
       g_pd3dDevice->SetStreamSource( 0, g_pVB2, vertSize5 * 2, sizeof(CUSTOMVERTEX) );
       g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX );        
       g_pd3dDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 0, g_cube_count2);
    }
   rotateCount[rotPos] = rotateCount[rotPos] - rotateInc;
   if(rotateCount[rotPos] < 0)
   {  
      rotateCount[rotPos] = D3DX_PI*2; 
   }
}

LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch( msg )
    {
        case WM_DESTROY:
            Cleanup();
            PostQuitMessage( 0 );
            return 0;
    }

    return DefWindowProc( hWnd, msg, wParam, lParam );
}


INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR cmdline, INT )
{
    DWORD    count = 0;
    DWORD    countPass = 0;
    char     amsg[512] = "";
    char     *p;
    char     *delims = {" ,"};
    char     commands[30][64];
    int      i, j;
    int      loopframes = 50;
    int      insecs = 5;
    int      intest = 1;
    int      inwidth = 640;
    int      inheight = 480;
    int      screenWide = 640;
    int      screenHigh = 480;
    int      xpos = 0;
    int      ypos = 0;
    BOOL     changeScreen = FALSE;
    BOOL     calibrate = TRUE;
    float    secs1frame;
    HDC      hdc;
    double   newPassSecs;
    double   oldPassSecs = 0;
    FILE     *chkfile = NULL;



    hInst = hInst;
    
    // Register the window class
    WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L, 
                      GetModuleHandle(NULL), NULL, NULL, NULL, NULL,
                      "D3D9 Benchmark", NULL };
    RegisterClassEx( &wc );

    // Create the application's window
    HWND hWnd = CreateWindow( "D3D9 Benchmark",
                             "D3D9 Benchmark ", 
                              WS_OVERLAPPEDWINDOW,
                              0,
                              0,
                              GetSystemMetrics(SM_CXSCREEN),
                              GetSystemMetrics(SM_CYSCREEN),
                              NULL, NULL, wc.hInstance, NULL );

    pixelsWide = GetSystemMetrics(SM_CXSCREEN);
    pixelsHigh = GetSystemMetrics(SM_CYSCREEN);
    screenWide = pixelsWide;
    screenHigh = pixelsHigh;

    StringCchCopy(imageFiles[0], 15, "tile1.bmp");
    StringCchCopy(imageFiles[1], 15, "tile2.bmp");
    StringCchCopy(imageFiles[2], 15, "tile4.bmp");
    StringCchCopy(imageFiles[3], 15, "atile256.bmp");
    StringCchCopy(imageFiles[4], 15, "windows.bmp");

    StringCchCopy(amsg, 250, cmdline);
    StringCchCopy(commandLine, 250, strupr(cmdline));
    i = 0;
    p = strtok(commandLine, delims);
    while (p != NULL)
    {
        StringCchCopy(commands[i], 30, p);
        i = i + 1;
        if (i > 40) break;
        p = strtok(NULL, delims);
    }
    if (i > 0)
    {
        StringCchCopy(commandLine, 250, amsg);
    }
    else
    {
        StringCchCopy(commandLine, 5, "None ");
    }
    autoRun = FALSE;

    for (j=0; j<i; j++)
    {
        switch(commands[j][0])
        {            
            case 'A': // Auto Run
             autoRun = TRUE;
             break;            

            case 'L': // log file
             if (j < i -1)
             {
                 sscanf(commands[j+1], "%s", logFile);
             }
             break;

            case 'S': // Seconds
             if (j < i -1)
             {
                 sscanf(commands[j+1], "%d", &insecs);
                 if (insecs < 5) insecs = 5; 
                 limitsecs = (double)insecs;
             }
             break;

            case 'T': // Test
             if (j < i -1)
             {
                 sscanf(commands[j+1], "%d", &intest);
                 if (intest < 1) intest = 1;
                 if (intest > 7) intest = 8; 
                 testToRun = intest;
                 lastTest  = intest; 
             }
             break;

            case 'W': // Pixel Width
             if (j < i -1)
             {
                 sscanf(commands[j+1], "%d", &inwidth);
                 if (inwidth < 100) inwidth = 100;
                 if (inwidth > 5000) inwidth = 5000;
                 changeScreen = TRUE; 
             }
             break;

            case 'H': // Pixel Height
             if (j < i -1)
             {
                 sscanf(commands[j+1], "%d", &inheight);
                 if (inheight < 100) inheight = 100;
                 if (inheight > 5000) inheight = 5000;
                 changeScreen = TRUE; 
             }
             break;

        }
    }
    for (j=0; j<i; j++)
    {
        
        if (strnicmp(commands[j],  "F1", 2) == 0)
        {
           sscanf(commands[j+1], "%s", imageFiles[0]);
        }
        if (strnicmp(commands[j],  "F2", 2) == 0)
        {
           sscanf(commands[j+1], "%s", imageFiles[1]);
        }
        if (strnicmp(commands[j],  "F3", 2) == 0)
        {
           sscanf(commands[j+1], "%s", imageFiles[2]);
        }
        if (strnicmp(commands[j],  "F4", 2) == 0)
        {
           sscanf(commands[j+1], "%s", imageFiles[3]);
        }
        if (strnicmp(commands[j],  "F5", 2) == 0)
        {
           sscanf(commands[j+1], "%s", imageFiles[4]);
        }
        if (strnicmp(commands[j],  "P1", 2) == 0)
        {
            xpos = 0;
            ypos = 0;
        }
        if (strnicmp(commands[j],  "P2", 2) == 0)
        {
            xpos = 0;
            ypos = screenHigh - inheight - 35;
            if (ypos < 0) ypos = 0;
        }
        if (strnicmp(commands[j],  "P3", 2) == 0)
        {
            xpos = screenWide - inwidth;
            if (xpos < 0) xpos = 0;
            ypos = 0;
        }
        if (strnicmp(commands[j],  "P4", 2) == 0)
        {
            xpos = screenWide - inwidth;
            if (xpos < 0) xpos = 0;
            ypos = screenHigh - inheight - 35;
            if (ypos < 0) ypos = 0;
        }
        if (strnicmp(commands[j],  "NOH", 3) == 0)
        {
            saveHeadings = FALSE;
        }
        if (strnicmp(commands[j],  "NOC", 3) == 0)
        {
            saveConfig = FALSE;
        }
    }
    if (changeScreen)
    {
         pixelsWide = inwidth;
         pixelsHigh = inheight;
         MoveWindow(hWnd, xpos, ypos, pixelsWide, pixelsHigh, TRUE);
    }
    chkfile = fopen("oneEffect.fx","r");  
    if (chkfile == NULL)
    {
       if (MessageBox(NULL, "Cannot open oneEffect.fx file", "Error",
               MB_ICONQUESTION | MB_OK) == IDOK)
       {
            UnregisterClass( "D3D9 Benchmark", wc.hInstance );
            return 0; 
       }
    }
    fclose(chkfile);
    chkfile = fopen("pixel.shd","r");  
    if (chkfile == NULL)
    {
       if (MessageBox(NULL, "Cannot open pixel.shd file", "Error",
               MB_ICONQUESTION | MB_OK) == IDOK)
       {
            UnregisterClass( "D3D9 Benchmark", wc.hInstance );
            return 0; 
       }
    }
    fclose(chkfile);
    chkfile = fopen("vertex.shd","r");  
    if (chkfile == NULL)
    {
       if (MessageBox(NULL, "Cannot open vertex.shd file", "Error",
               MB_ICONQUESTION | MB_OK) == IDOK)
       {
            UnregisterClass( "D3D9 Benchmark", wc.hInstance );
            return 0; 
       }
    }
    fclose(chkfile);

    StringCchCopy(testTitle[1], 60, " Egg Gouraud shading");
    StringCchCopy(testTitle[2], 60, " Egg Wireframe, Vsync");
    StringCchCopy(testTitle[3], 60, " Wireframe 500 Cubes");
    StringCchCopy(testTitle[4], 60, " Textured Tunnel");
    StringCchCopy(testTitle[5], 60, " Plain Colour Objects");
    StringCchCopy(testTitle[6], 60, " Textured Objects");
    StringCchCopy(testTitle[7], 60, " Pixel Shader 2.0");
    StringCchCopy(testTitle[8], 60, " Vertex Shader 2.0");
    StringCchCopy(testTitle[9], 60, " Finished");

    hdc = GetDC(hWnd);
    bpp = GetDeviceCaps(hdc, BITSPIXEL);
    ReleaseDC(hWnd, hdc);
    getDetails();
     
    keyPress = GetAsyncKeyState(VK_ESCAPE);

    if (!autoRun)
    {
        if (!changeScreen)
        {
            StringCchPrintf(amsg, 300, "Commands: %s\n\n"
               "At end, results should be in file %s  \n\n"
               "Test at full screen window size %d x %d x %d bits  \n"
               "%d Seconds per test, Tests %d to %d  \n\n"
               "Press OK to run or CANCEL to exit. For run time  \n" 
               "command line options see (Edit) runVidD3D9.bat  \n\n", 
                commandLine,  logFile, pixelsWide, pixelsHigh,
                bpp, (int)limitsecs, testToRun, lastTest);
        }
        else
        {
            StringCchPrintf(amsg, 300, "Command: %s\n\n"
               "At end, results should be in file %s  \n\n"
               "Test at screen window size %d x %d x %d bits  \n\n"
               "%d Seconds per test, Tests %d to %d  \n\n"
               "Press OK to run or CANCEL to exit.  \n\n", 
                commandLine,  logFile, pixelsWide, pixelsHigh,
                 bpp, (int)limitsecs, testToRun, lastTest);
        }
        int mbox = MessageBox(NULL, amsg, "VideoD3D9 DirectX 9 Speed Tests", MB_ICONINFORMATION | MB_OKCANCEL);
        if (mbox == IDCANCEL)
        {
            UnregisterClass( "D3D9 Benchmark", wc.hInstance );
            return 0; 
        }
    }
    outfile = fopen(logFile,"a+");  
    if (outfile == NULL)
    {
       if (MessageBox(NULL, "Cannot open results file", "Error",
               MB_ICONQUESTION | MB_OK) == IDOK)
       {
            UnregisterClass( "D3D9 Benchmark", wc.hInstance );
            return 0; 
       }
    }
    local_time();    
    if (saveHeadings)
    {
       fprintf(outfile, "\n\n ###########################################################################\n");
       fprintf(outfile, "      DirectX9 D3D Test %s, %s", version, timeday);
       fprintf(outfile, "                      Copyright (C) 2006, Roy Longbottom\n\n");
    }
    aspect=(float)pixelsWide / (float)pixelsHigh;
    aspc = (aspect - 1.0f) / 2.0f + 1.1f;
    vEyePt2.z = vEyeOut;

    for (i=0; i<50; i++)
    {
        rotateCount[i] = 0.0f;
    }
    // Enter the message loop
    MSG msg; 
    ZeroMemory( &msg, sizeof(msg) );
    while( msg.message!=WM_QUIT )
    {
        if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
        {
            TranslateMessage( &msg );
            DispatchMessage( &msg );
        }
        else
        {
            if (phase == 1)
            {
                // Initialize Direct3D
                if( SUCCEEDED( InitD3D( hWnd ) ) )
                {
                    // Create the scene geometry
                    if( SUCCEEDED( InitGeometry() ) )
                    {
                        if (testToRun > 2)
                        {
                            if( SUCCEEDED( InitVB() ) )
                            {
                                // Show the window
                                g_pd3dDevice->SetRenderState(D3DRS_LIGHTING,FALSE);
                                ShowWindow( hWnd, SW_SHOWDEFAULT );
                                UpdateWindow( hWnd );
                                count = 0;
                                countPass = 0;
                                phase = 2;
                                ShowCursor(FALSE);  
                                start_time();
                            }
                            else
                            {
                                MessageBox(NULL, "InitVB Failed", "ERROR", MB_OK);
                                fprintf(outfile, "\n InitVB Failed\n\n");
                                fflush(outfile);
                                fclose(outfile);
                                UnregisterClass( "D3D9 Benchmark", wc.hInstance );
                                return 0; 
                            }
                        }
                        else
                        {
                            // Show the window
                            ShowWindow( hWnd, SW_SHOWDEFAULT );
                            UpdateWindow( hWnd );
                            count = 0;
                            countPass = 0;
                            phase = 2;
                            ShowCursor(FALSE);  
                            start_time();
                        }
                    }
                    else
                    {
                        MessageBox(NULL, "InitGeometry Failed", "ERROR", MB_OK);
                        fprintf(outfile, "\n InitGeometry Failed\n\n");
                        fflush(outfile);
                        fclose(outfile);
                        UnregisterClass( "D3D9 Benchmark", wc.hInstance );
                        return 0; 
                    }               
                }
                else
                {
                    MessageBox(NULL, "InitD3D Failed", "ERROR", MB_OK);
                    fprintf(outfile, "\n InitD3D Failed\n\n");
                    fflush(outfile);
                    fclose(outfile);
                    UnregisterClass( "D3D9 Benchmark", wc.hInstance );
                    return 0;
                }
                calibrate = TRUE; 
            }
            else if (phase == 2)
            {
                if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
                {
                    fprintf(outfile, "\n ESCAPE PRESSED\n\n");
                    fflush(outfile);
                    fclose(outfile);
                    UnregisterClass( "D3D9 Benchmark", wc.hInstance );
                    return 0; 
                }
                if (testToRun == 7 && !pixelShaderV2)
                {
                    phase = 3;
                }
                else if (testToRun == 8 && !vertexShaderV2)
                {
                    phase = 3;
                }
                else
                {         
                    if (calibrate)
                    {
                        Render();
                        start_time();
                        Render();
                        end_time();
                        fps =  1.0 / secs;
                        StringCchPrintf(amsg, 100, "   %s                   ", testTitle[testToRun]);
                        SetWindowText(hWnd, amsg);
                        loopframes = (int)(0.05 / secs);
                        if (loopframes < 1) loopframes = 1;
                        rotateAdj = 100.0f / (float)loopframes;
                        if (loopframes > 100) loopframes = 100;
                        start_time();
                    }
                    for (i=0; i<loopframes; i++)
                    {
                        Render();
                    }
                    count = count + loopframes;
                    countPass = countPass + loopframes;
                    end_time();
                    if (runOne)
                    {
                        newPassSecs = secs - oldPassSecs;
                        if (newPassSecs >= passSecsLimit)
                        {
                            oldPassSecs = secs;
                            Passfps = (double)countPass / newPassSecs;
                            countPass = 0;
                            fprintf (outfile, "%7.1f ", Passfps);
                            fprintf (outfile, "Frames Per Second over %d seconds\n", (int)passSecsLimit);
                        }
                    }
                    fps =  (double)count / secs;
                    if (calibrate)
                    {
                        if (testToRun == 4)
                        {
                            secs1frame = (vEyeIn - vEyeOut) / vEyeInc / (float) fps; 
                            vEyeInc = vEyeInc * secs1frame;
                            count = 0;
                            start_time();                    
                        }
                        calibrate = FALSE;
                    }
                    fps =  (double)count / secs;
                    StringCchPrintf(amsg, 150, " %d x %d x %d bits,   %s,   %8.1f Frames Per Second,   %d out of %d Seconds",
                                 pixelsWide, pixelsHigh, bpp, testTitle[testToRun], fps, (int)secs, (int)limitsecs);
                    SetWindowText(hWnd, amsg);
                    if (secs > limitsecs) phase = 3;
                }             
            }
            else if (phase == 3)
            {
                results[testToRun] = fps;
                if (runOne)
                {
                    if (testToRun == 7) 
                    {
                        if (pixelShaderV2)
                        {
                            fprintf (outfile, "%7.1f ", results[testToRun]);
                            fprintf (outfile, "Frames Per Second Overall\n");
                        }
                        else
                        {
                            fprintf (outfile, "  Pixel Shader Not Available\n");
                            ShowCursor(TRUE);
                            MessageBox(NULL, "Pixel Shader 2 Not Available", "ERROR", MB_OK);
                        }
                    }
                    else if (testToRun == 8) 
                    {
                        if (vertexShaderV2)
                        {
                            fprintf (outfile, "%7.1f ", results[testToRun]);
                            fprintf (outfile, "Frames Per Second Overall\n");
                        }
                        else
                        {
                            fprintf (outfile, "  Vertex Shader Not Available\n");
                            ShowCursor(TRUE);
                            MessageBox(NULL, "Vertex Shader 2 Not Available", "ERROR", MB_OK);
                        }
                    }
                    else
                    {
                        fprintf (outfile, "%7.1f ", results[testToRun]);
                        fprintf (outfile, "Frames Per Second Overall\n");
                    }
                }
                else
                {
                    if (testToRun == 7) 
                    {
                        if (pixelShaderV2)
                        {
                            fprintf (outfile, "%7.1f ", results[testToRun]);
                        }
                        else
                        {
                            ShowCursor(TRUE);
                            fprintf (outfile, "PS2 N/A ");
                        }
                    }
                    else if (testToRun == 8) 
                    {
                        if (vertexShaderV2)
                        {
                            fprintf (outfile, "%7.1f ", results[testToRun]);
                        }
                        else
                        {
                            ShowCursor(TRUE);
                            fprintf (outfile, "VS2 N/A ");
                        }
                    }
                    else
                    {
                        fprintf (outfile, "%7.1f ", results[testToRun]);
                    }
                }
                ShowCursor(TRUE);
                testToRun = testToRun + 1;
                Cleanup();
                phase = 1;
                if (testToRun == lastTest + 1) phase = 4;
                StringCchPrintf(amsg, 30, "   %s ", testTitle[testToRun]);
                SetWindowText(hWnd, amsg);
            }
            else
            {
                if (runOne)
                {
                    fprintf (outfile, "\n");
                }
                else
                {
                    if (saveConfig)
                    {
                        fprintf (outfile, "\n\n");
                    }
                    else
                    {
                        fprintf (outfile, "\n");
                    }
                }
                if (saveConfig)
                {
                    fprintf(outfile, "  Test Seconds %d\n", (int)limitsecs);
                    if (strnicmp(commandLine,  "None", 4) != 0)
                    {
                        fprintf(outfile, "  Commands: %s\n", commandLine);
                    }
                    fprintf(outfile, "  Adapter %s\n", thisAdapter);
                    for (i=0; i<9; i++)
                    {
                        fprintf(outfile, "%s\n", configdata[i]);
                    }
                    fprintf (outfile, "\n");          
                }
                fflush(outfile);
                fclose(outfile);
                UnregisterClass( "D3D9 Benchmark", wc.hInstance );
                return 0;
            }
        }
    }
    UnregisterClass( "D3D9 Benchmark", wc.hInstance );
    return 0;
} // WinMain


void local_time()
{
    time_t time_of_day;
    time_of_day = time( NULL );
    StringCchCopy(timeday, 30, ctime(&time_of_day)); // timeday = date and time
    return;
}

