/*
 *  OneDD VideoDD1 DirectDraw Benchmark Copyright (C) Roy Longbottom 2004
 *
 *  Partly based on example programs, Copyright (C) 1995-1996 Microsoft
 *  Corporation
 *  
 *  OneDD.exe runs the benchmark either stand-alone at existing screen
 *  settings or varying settings via VideoDD1.exe. Results are saved
 *  in file VideoLog.txt. The following tests are run :
 *
 *  testPass
 *
 *    0 -    Bltfast video RAM, Test1()
 *    1 -    Blt with stretch, Test2()
 *    2 -    Blt Fill rectangle, Test3and4()
 *    3 -    As 2 with WaitForVerticalBlank 
 *    4 -    Bltfast from CPU RAM, Test5()         
 *
 *  Speed is measured in Millions of Pixels Per Second plus Frames
 *  Per Second on testPass 0, 4 and 3 (for monitor refresh Hz).
 *
 *  Modules
 *
 *  config.cpp - via getDetails() identifies CPU type/speed, memory
 *            size, OS version, also running times via start_time()
 *            and end_time() plus local_time() for date and clock time
 *  OneDD.rc - Only icon and bmp file IDs
 *  OneDD.h, OneDD.ico
 *  B128x64.bmp - Two 64 x 64 pixel x 8 bit images loaded to lpDDSOne
 *  B256x256.bmp - 256 x 256 pixel x 8 bit image loaded to lpDDSTwo
 *
 *  DDUtil.CPP, DDUtil.h - Copyright (C) 1995-1996 Microsoft Corporation
 *            Contains DDLoadBitmap(), DDReLoadBitmap(), DDLoadPalette(),
 *            DDSetColorKey() and other unused functions.
 *
 *  OneDD.cpp
 *
 *  Particularly with Windows XP, on running a benchmark that displays
 *  on-going results, the window for the latter can be covered by
 *  another window. To overcome this, all tests are run via the WinMain 
 *  message processing loop using the testPass variable.
 *
 *  Functions
 *
 *  WinMain() - usual Windows functions, runs setUpTests(), runTests()
              and EndTests()
 *  InitApp() - usual Windows functions, processes command line,
 *            allocates variable values, obtains number of video
 *            drivers to enable choice of second video card using
 *            DDEnumCallback()
 *  WindowProc() - usual Windows functions for msgs, not used
 *  DDEnumCallback() - gets capabilities of all DD drivers
 *  setUpTests() - opens log file, allocates some values, runs setUp()
 *  setUp() - creates the main DirectDraw object, primary surface,
 *            palette, offscreen surface, system memory area, loads
 *            bitmap and sets color key
 *  runTests() - runs one of Test1() to Test5() once then calibrates
 *            runPasses, this number being repeated for runTime.
 *            progress and speed details is also displayed.  
 *  Test1() - Fills the screen with 64 x 64 pixels images from the two
 *            in B128x64.bmp offscreen surface lpDDSOne alternately,
 *            copying to the same area for runPasses. Next repeat
 *            images are reversed (rcRect2 before rcRect).
 *  Test2() - Selects a 128 x 128 pixel square rcRect2 screen top left 
 *            and copies one of images from lpDDSOne to it, stretching.
 *            Loops increase rcRect2 width and height up to window
 *            height and left position to move across the screen.
 *  Test3and4() - Use Blt with COLORFILL to paint different coloured
 *            stripes, darkening on each pass. Test4 (2 second test)
 *            calls waitSynch() to synchronize painted frame speed to
 *            no more than screen refresh Hz.
 *  Test5() - SYSTEMMEMORY area lpDDSBig is created as screenWide x
 *            screenWide pixels. B256x256.bmp image in lpDDSTwo is
 *            stretched to fill lpDDSBig. The test scrolls lpDDSBig
 *            down and up the screen.
 *  waitSynch() - WaitForVerticalBlank() - option for all tests if
 *            variable waitForSynch = TRUE.
 *
 *            At the end of each Test() the following is included:
 *            gotPixel = gotPixel | GetPixel(hdc, 2, 1);
 *            intended to overcome the problem of cached graphics
 *            commands which are executed asynchronously when testCount
 *            can be huge relative to runTime.
 *
 *  EndTests() - Saves results to the log file and runs finiObjects().
 *  finiObjects() - Releases DD objects and causes program exit. 
 *  restoreAll() - Restores DD objects if SURFACELOST.
 *  initFail() - Saves details of any detected errors to the log file. 
 *
 *   ************************* WARNING ****************************** 
 *   ddraw.lib MUST BE INCLUDED in compiler Link Library modules.
 */

 
#define NAME "DDTest"
#define TITLE " Roy Longbottom's Direct Draw Function Timingf Test "

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include <commdlg.h>
#include <ddraw.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <time.h>

#include "ddutil.h"
#include "asmtime.h"



//  ************** PROTOPTYPES *****************

LRESULT CALLBACK WindowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);                 

BOOL FAR PASCAL DDEnumCallback( LPGUID, LPSTR, LPSTR, LPVOID);
static BOOL InitApp(HINSTANCE hInstance, LPSTR cmdline, int cmdshow );

HRESULT restoreAll();
static void finiObjects();
BOOL setUp(HWND);
BOOL setUpTests(HWND);
BOOL runTests();
BOOL EndTests();
BOOL initFail(HWND);
BOOL waitSynch();
BOOL Test1();
BOOL Test2();
BOOL Test3and4();
BOOL Test5();
void local_time();

#define version "Version 4.1x for 64 bit Windows"
// #define version "Version 4.1x for 32 bit Windows"

LPDIRECTDRAW            lpDD = NULL;           // DirectDraw object
LPDIRECTDRAWSURFACE     lpDDSPrimary = NULL;   // DirectDraw primary surface
LPDIRECTDRAWSURFACE     lpDDSOne = NULL;       // Offscreen surface 1
LPDIRECTDRAWSURFACE     lpDDSTwo = NULL;       // Offscreen surface 1
LPDIRECTDRAWSURFACE     lpDDSBig = NULL;       // System memory surface
LPDIRECTDRAWPALETTE     lpDDPal = NULL;        // DirectDraw palette

#define MAX_DRIVERS 5   // maximum D3D drivers we ever expect to find

struct _v      // for display adapter drivers
{
    LPGUID lpGUID[MAX_DRIVERS];
    int  NumVDrivers;
    GUID  VDriverGUID[MAX_DRIVERS];
    char VDriverName[MAX_DRIVERS][50];
    char VDriverDesc[MAX_DRIVERS][100];
} v;

HANDLE    MyInstance;
DDBLTFX   ddbl;
FILE      *outfile = NULL;
HWND      mainHWND;

DWORD   gotPixel;

BOOL    escPress = FALSE;
BOOL    bQuit = FALSE;
BOOL    saveHeadings = TRUE;
BOOL    saveResults = TRUE;
BOOL    saveEnd = TRUE;
BOOL    autoRun = FALSE;
BOOL    waitForSynch = FALSE;
BOOL    reverse;
BOOL    oddCol;
BOOL    oddColStart;
BOOL    noGo;

RECT    winRect;
RECT    rcScreen;
RECT    rcRect;
RECT    rcRect2;

double  speed;
double  mbytes;
double  testCount;
double  pass1Secs[11];
double  framesPerSec[11];
double  mpixPerSec[11];

int     currentVDriver = 0;
int     screenWide;
int     screenHigh;
int     bitsaPixel;
int     picWide, picHigh, picLeft;
int     picTop = 30;
int     testPass;
int     configs = 40;
int     maxTests = 11;
int     keyPress;
int     bitmapRepeats = 4;
int     runTime = 5;
int    bigSize;
int    runPasses;
int    pass1Passes[11];
int    j1, k1;
int    k1inc;
int    i1 = 255;
int    i2 = 0;
int    i3 = 10;
int    i4 = 0;
int    rt, rb, ri;
int    sceeenInc;
int    firstTest = 0;
int    lastTest  = 4;

char    failMessage[100];
char    testTitle[11][40] = {" "};
char    logFile[100] = "VideoLog.txt";
char    commandLine[256] = "";
char    note4[600] = " Commands ";
char    testres[200];
char    szBitmap1[10] = "B128x64";
char    szBitmap2[10] = "B256x256";
char    heading1[11][8];
char    heading2[11][8];
char    timeday[30];


// restore all lost objects

HRESULT restoreAll( void )
{
    HRESULT     ddrval;

    ddrval = lpDDSPrimary->Restore();
    if( ddrval == DD_OK )
    {
        ddrval = lpDDSOne->Restore();
        if( ddrval == DD_OK )
        {
            ddrval = lpDDSTwo->Restore();
            if( ddrval == DD_OK )
            {
                DDReLoadBitmap(lpDDSTwo, szBitmap2);
                ddrval = lpDDSBig->Restore();
                if( ddrval == DD_OK )
                {
                    DDReLoadBitmap(lpDDSOne, szBitmap1);
                }
            }
        }
    }
    return ddrval;

} // restoreAll


BOOL runTests()
{
    HRESULT  ddrval;
    HBRUSH hbr;

    HDC      hdc;
    BOOL     firstPass = TRUE;
    BOOL     passTime = FALSE;
    int      passSecs;

    gotPixel = 0;
    GdiFlush();

    testCount = 0.0;
    reverse = FALSE;
    oddCol = TRUE;
    oddColStart = TRUE;
        
    rcRect.left = 0;
    rcRect.top = 0;
    rcRect.right = 256;
    rcRect.bottom = 256;

    rcRect2.left = 0;
    rcRect2.top = 0;
    rcRect2.right = bigSize;
    rcRect2.bottom = bigSize;

    if (lpDDSPrimary->GetDC(&hdc) == DD_OK)
    {
       hbr = CreateSolidBrush(RGB(0, 0, 0));  
       SelectObject(hdc, hbr);
       Rectangle(hdc, 0, 0, screenWide, screenHigh);
       DeleteObject(hbr);

       SetBkColor(hdc, RGB(0, 0, 0));
       SetTextColor(hdc, RGB(255, 0, 0)); 
       sprintf (testres, "%dx%dx%d bits %s                                                         ",
                   screenWide, screenHigh, bitsaPixel, testTitle[testPass]);
       TextOut(hdc, 10,  1, testres, lstrlen(testres));
       lpDDSPrimary->ReleaseDC(hdc);
    }
            
    //  copy to lpDDSBig

    if(testPass == 4)
    {
         while( 1 )
         {
            ddrval = lpDDSBig->Blt( &rcRect2, lpDDSTwo, &rcRect, DDBLT_WAIT , &ddbl);     
            if( ddrval == DD_OK )
            {
                break;
            }
            if( ddrval == DDERR_SURFACELOST )
            {
                 ddrval = restoreAll();
                 if( ddrval != DD_OK )
                  {
                      sprintf(failMessage, "restoreAll 1 FAILED");
                      return initFail(mainHWND);
                  }
            }
            if( ddrval != DDERR_WASSTILLDRAWING )
            {
                return TRUE;
            }
         }
         if(ddrval != DD_OK)
         {
             sprintf(failMessage, "ddrval != DD_OK 1 FAILURE");
             return initFail(mainHWND);
         }
    }

    // size of one image
    
    rcRect.left = 0;
    rcRect.top = 0;
    rcRect.right = 64;
    rcRect.bottom = 64;
    
    rcRect2.left = 64;
    rcRect2.top = 0;
    rcRect2.right = 128; 
    rcRect2.bottom = 64;

    picWide = (screenWide) / 128;
    picWide = picWide * 2;
    picLeft = (screenWide - picWide * 64) / 2;     
    picHigh = (screenHigh - picTop) / 64;                 

    runPasses = 1;

    if (testPass == 0)
    {
        runPasses = 2;
        oddColStart = TRUE;
        if (!Test1()) return FALSE;
        if (noGo) return TRUE;
        runPasses = 1;
        oddColStart = FALSE;
        testCount = 0;

        start_time();
        do
        {
            if (oddColStart)
            {
                oddColStart = FALSE;
            }
            else
            {
                oddColStart = TRUE;
            }
            if (!Test1()) return FALSE;
            testCount = testCount + runPasses;
            pass1Passes[testPass] = runPasses;
           end_time();
           if (!passTime)
           {
               pass1Secs[testPass] = secs / runPasses;
               passTime = TRUE;
           }
           if (firstPass)
           {                   
               if (secs < 0.001)
               {
                   runPasses = runPasses * 10;
               }
               else
               {
                    runPasses = (int)(runPasses / secs); // /2
                    if (runPasses < 1) runPasses = 1;
                    firstPass = FALSE;
               }                   
           }
           speed = 0;
           framesPerSec[testPass] = 0;    
           if (secs > 0)
           {
                speed = (double)(picHigh * picWide) * 64 * 64 *
                           testCount / secs / 1000000.0;
                framesPerSec[testPass] = testCount / secs;
           }
           if (lpDDSPrimary->GetDC(&hdc) == DD_OK)
           {
               hbr = CreateSolidBrush(RGB(0, 0, 0));  
               SelectObject(hdc, hbr);
               Rectangle(hdc, 0, 0, screenWide, picTop);
               DeleteObject(hbr);

               SetBkColor(hdc, RGB(0, 0, 0));
               SetTextColor(hdc, RGB(255, 0, 0)); 
               sprintf (testres, "%dx%dx%d bits %s %8.1fM Pixels Per Sec %5.1f Secs                        ",
                                       screenWide, screenHigh, bitsaPixel, testTitle[testPass], speed, secs);
               TextOut(hdc, 10,  1, testres, lstrlen(testres));
               lpDDSPrimary->ReleaseDC(hdc);
           }        
           if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
           {
               testPass = lastTest + 1;
               escPress = TRUE;
               return TRUE;
           }
        }
        while(secs < runTime);

        mpixPerSec[testPass] = speed;
   }
   else if (testPass == 1)
   {
      rcRect.left = 0;
      rcRect.top = 0;
      rcRect.right = 64;
      rcRect.bottom = 64;
    
      j1 = -50;
      k1 = 1;
      k1inc = 1;
      start_time();
      if (!Test2()) return FALSE;
      if (noGo) return TRUE;
      end_time();

      if (secs > 0.004) k1inc = (int)((double)k1inc * secs / 0.004);
      j1 =  -50;
      k1 = screenHigh - 160;
      testCount = 0;

      // ************ Test 2  testPass 1 Blt Stretch                     

      start_time();
      do
      {
           if (!Test2()) return FALSE;
           pass1Passes[testPass] = runPasses;
           end_time();
           if (!passTime)
           {
               pass1Secs[testPass] = secs;
               passTime = TRUE;
           }
           if (firstPass)
           {                   
               if (secs < 0.001)
               {
                   runPasses = runPasses * 10;
               }
               else
               {
                    runPasses = (int)(runPasses / secs / 2);
                    if (runPasses < 1) runPasses = 1;
                    firstPass = FALSE;
               }
           }
           speed = 0;

           if (secs > 0)
           {
              speed = testCount / secs / 1000000.0;               
           }
           if (lpDDSPrimary->GetDC(&hdc) == DD_OK)
           {
               hbr = CreateSolidBrush(RGB(0, 0, 0));  
               SelectObject(hdc, hbr);
               Rectangle(hdc, 0, 0, screenWide, picTop);
               DeleteObject(hbr);

               SetBkColor(hdc, RGB(0, 0, 0));
               SetTextColor(hdc, RGB(255, 0, 0));
               sprintf (testres, "%dx%dx%d bits %s %8.1fM Pixels Per Sec %5.1f Secs                                ",
                           screenWide, screenHigh, bitsaPixel, testTitle[testPass], speed, secs);
               TextOut(hdc, 10,  1, testres, lstrlen(testres));
               lpDDSPrimary->ReleaseDC(hdc);
           }
           if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
           {
               testPass = lastTest + 1;
               escPress = TRUE;
               return TRUE;
           }
      }
      while(secs < runTime);

      mpixPerSec[testPass] = speed;              
   }
   else if (testPass == 2 || testPass == 3)
   {
        // ************ Test 3  testPass 2 Blt Fill rectangle             
        // ************ Test 4  testPass 3 Blt WaitForVerticalBlank

      rcScreen.left   = 0;
      rcScreen.right  = screenWide;
      sceeenInc = (screenHigh - picTop) / 8;                    

      passSecs = runTime;
      if (testPass == 3) passSecs = 2;

      if (!Test3and4()) return FALSE;
      if (noGo) return TRUE;
      testCount = 0;
      
      start_time();
      do
      {
           if (!Test3and4()) return FALSE;
           pass1Passes[testPass] = runPasses;
           end_time();
           if (!passTime)
           {
               pass1Secs[testPass] = secs;
               passTime = TRUE;
           }
           if (firstPass)
           {
               if (secs < 0.001)
               {
                   runPasses = runPasses * 10;
               }
               else
               {
                  runPasses = (int)(runPasses / secs);
                  if (runPasses < 1) runPasses = 1;
                  firstPass = FALSE;
               }
           }
           speed = 0;
           framesPerSec[testPass] = 0;               
           if (secs > 0)
           {
                speed = (double)screenWide * (double)(screenHigh - picTop) * 
                      testCount / secs / 1000000.0;
                framesPerSec[testPass] = testCount / secs;
           }
           if (lpDDSPrimary->GetDC(&hdc) == DD_OK)
           {
               hbr = CreateSolidBrush(RGB(0, 0, 0));  
               SelectObject(hdc, hbr);
               Rectangle(hdc, 0, 0, screenWide, picTop);
               DeleteObject(hbr);

               SetBkColor(hdc, RGB(0, 0, 0));
               SetTextColor(hdc, RGB(255, 0, 0));
               sprintf (testres, "%dx%dx%d bits %s %8.1fM Pixels Per Sec %5.1f Secs                                   ",
                                  screenWide, screenHigh, bitsaPixel, testTitle[testPass], speed, secs);
               if(testPass == 3) sprintf (testres, "%s %8.1f Frames Per Second %5.1f Seconds                                 ",
                                                        testTitle[testPass], framesPerSec[testPass], secs);
               
               TextOut(hdc, 10,  1, testres, lstrlen(testres));
               lpDDSPrimary->ReleaseDC(hdc);
           }
           if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
           {
               testPass = lastTest + 1;
               escPress = TRUE;
               return TRUE;
           }
      }
      while(secs < passSecs);

      mpixPerSec[testPass] = speed;
   }
   else if (testPass == 4)
   {
      // ************ Test 4 testPass 4 BltFast CPU memory no back buffer      

        ri = screenWide - screenHigh + picTop;
        i4 = screenWide * screenHigh / 100000 * bitsaPixel / 32;
        if (i4 < 1) i4 = 1;
        rcRect.left = 0;
        rcRect.right = screenWide;

        rt = 0;
        rb = screenHigh - picTop;
        reverse = FALSE;
        if (!Test5()) return FALSE;
        if (noGo) return TRUE;
        
        rt = 0;
        rb = screenHigh - picTop;
        reverse = FALSE;
        testCount = 0;

        start_time();  
        do
        {
           if (!Test5()) return FALSE;
           pass1Passes[testPass] = runPasses;
           end_time();
           if (!passTime)
           {
               pass1Secs[testPass] = secs;
               passTime = TRUE;
           }
           if (firstPass)
           {     
               if (secs < 0.001)
               {
                   runPasses = runPasses * 10;
               }
               else
               {
                   runPasses = (int)(runPasses / secs / 2);
                   if (runPasses < 1) runPasses = 1;
                   firstPass = FALSE;
               }
           }
           speed = 0;
           framesPerSec[testPass] = 0;
           if (secs > 0)
           {
                speed = screenWide * (screenHigh - picTop) * testCount / secs / 1000000.0;
                framesPerSec[testPass] = testCount / secs;
           }
           if (lpDDSPrimary->GetDC(&hdc) == DD_OK)
           {
               hbr = CreateSolidBrush(RGB(0, 0, 0));  
               SelectObject(hdc, hbr);
               Rectangle(hdc, 0, 0, screenWide, picTop);
               DeleteObject(hbr);

               SetBkColor(hdc, RGB(0, 0, 0));
               SetTextColor(hdc, RGB(255, 0, 0)); 
               sprintf (testres, "%dx%dx%d bits %s %8.1fM Pixels Per Sec %5.1f Secs                        ",
                                       screenWide, screenHigh, bitsaPixel, testTitle[testPass], speed, secs);
               TextOut(hdc, 10,  1, testres, lstrlen(testres));
               lpDDSPrimary->ReleaseDC(hdc);
           }        
           if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
           {
               testPass = lastTest + 1;
               escPress = TRUE;
               return TRUE;
           }
        }
        while(secs < runTime);

        mpixPerSec[testPass] = speed;
   }
   GdiFlush();
   if (GetAsyncKeyState(VK_ESCAPE) != keyPress || escPress == TRUE)
   {
        testPass = lastTest + 1;
        escPress = TRUE;
   }
   return TRUE;
                     
}  // runTests


// finiObjects
// finished with all objects we use;  them

static void finiObjects( void )
{
    if( lpDD != NULL )
    {
        if( lpDDSPrimary != NULL )
        {
            lpDDSPrimary->Release();
            lpDDSPrimary = NULL;
        }
        if( lpDDSOne != NULL )
        {
            lpDDSOne->Release();
            lpDDSOne = NULL;
        }
        if( lpDDSTwo != NULL )
        {
            lpDDSTwo->Release();
            lpDDSTwo = NULL;
        }
        if( lpDDSBig != NULL )
        {
            lpDDSBig->Release();
            lpDDSBig = NULL;
        }
        if( lpDDPal != NULL )
        {
            lpDDPal->Release();
            lpDDPal = NULL;
        }
        lpDD->Release();
        lpDD = NULL;
    }
    ShowCursor(TRUE);
    bQuit = TRUE;

} // finiObjects


// WindowProc - handle messages for the main application window

LRESULT CALLBACK  WindowProc(HWND hwnd, UINT msg,
                            WPARAM wparam, LPARAM lparam)                  
{     
    switch( msg )
    {
    case WM_ACTIVATEAPP:
        break;

        
    case WM_CREATE:
        break;

    case WM_DESTROY:
        finiObjects();
        bQuit = TRUE;
        break;

    case WM_COMMAND:
            break;

    case WM_PAINT:
        break;
           

    case WM_ACTIVATE:
         break;
    } 
    return DefWindowProc( hwnd, msg, wparam, lparam );

} /* WindowProc */





// This function is called if the initialization function fails

BOOL initFail( HWND hwnd )
{
   if (outfile != NULL)
   {
       fprintf (outfile, "Resolution %d x %d x %d bits/pixel\n",
                             screenWide, screenHigh, bitsaPixel);
       fprintf (outfile, "ERROR MESSAGE %s \n\n", failMessage);
       fflush (outfile);
       fclose(outfile);
   }
   finiObjects();
   ShowCursor(TRUE);  
   if (!autoRun) MessageBox( hwnd, failMessage, TITLE, MB_OK );
   DestroyWindow( hwnd );
   return FALSE;

} // initFail


// InitApp - register window class for the application,
//                 and do any other application initialization

static BOOL InitApp(HINSTANCE hInstance, LPSTR cmdline, int cmdshow )
{
    HWND                hwnd;
    WNDCLASS            wc;
    HDC hdc;
    char               mbuff[256];
    
    int   j;              
    cmdshow = cmdshow;
    
    /*
     * set up and register window class
     */
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof( DWORD );
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon( hInstance, "GenericIcon" );
    wc.hCursor = LoadCursor( NULL, IDC_ARROW );
    wc.hbrBackground = (HBRUSH)GetStockObject( BLACK_BRUSH );
    wc.lpszMenuName = "GenericMenu";
    wc.lpszClassName = NAME;
    if (!RegisterClass(&wc)) return FALSE;

    /*
     * create main window
     */

    hwnd = CreateWindowEx(
        0,
        NAME,  
        TITLE,  
        WS_POPUP  | WS_MAXIMIZE, // | WS_CAPTION | WS_SYSMENU,
        0,
        0,
        GetSystemMetrics(SM_CXSCREEN),
        GetSystemMetrics(SM_CYSCREEN),
        NULL,
        NULL,
        hInstance,
        NULL );

                    
    if( !hwnd ) return( FALSE );

    mainHWND = hwnd;

    strcpy(commandLine, strupr(cmdline));
    if (strlen(commandLine) > 1)
    {
        strcat(note4, commandLine);
    }
    else
    {
        strcpy(note4, "");
    }
 
    /*
     * set up data associated with this window
     */

    hdc = GetDC(hwnd);
    bitsaPixel = GetDeviceCaps(hdc, BITSPIXEL);
    screenWide = GetDeviceCaps(hdc, HORZRES);    
    screenHigh = GetDeviceCaps(hdc, VERTRES);
    ReleaseDC(hwnd, hdc);

    mbytes = (double)bitmapRepeats * 384.0 / 1024.0 * 640.0 / 1024.0;


    strcpy(heading1[0], "bltfast");
    strcpy(heading2[0], "vid_ram");
    strcpy(heading1[1], "blt_and");
    strcpy(heading2[1], "stretch");
    strcpy(heading1[2], "bltfill");
    strcpy(heading2[2], "rectngl");
    strcpy(heading1[3], "bltrect");
    strcpy(heading2[3], "+vsynch");
    strcpy(heading1[4], "bltfast");
    strcpy(heading2[4], "cpu_ram");
    strcpy(heading1[5], "+vsynch");
    strcpy(heading2[5], "    FPS");
    strcpy(heading1[6], "vid_ram");
    strcpy(heading2[6], "    FPS");
    strcpy(heading1[7], "cpu_ram");
    strcpy(heading2[7], "    FPS");


    keyPress = GetAsyncKeyState(VK_ESCAPE);

    // Identify details of drivers for all graphics adapters
    
    v.NumVDrivers = 0;
    if (DirectDrawEnumerate(DDEnumCallback, &v) != DD_OK)
    {
        printf(failMessage, "DirectDrawEnumerate FAILED");
        return initFail(mainHWND);
    }
    if(v.NumVDrivers == 0)
    {
        printf(failMessage, "Display Driver not found");
        return initFail(mainHWND);
    }

     
    // command line scan
    if (strlen(commandLine))
    {
       for (j=0; j<lstrlen(commandLine); j++)
       {
          if (strnicmp(&commandLine[j],  "secs ", 8) == 0)
          {
              sscanf(&commandLine[j+8], "%d", &runTime);
              if (runTime < 1) runTime = 1;
              if (runTime > 900) runTime = 900;
          }
          if (strnicmp(&commandLine[j],  "RUN", 3) == 0)
          {
              autoRun = TRUE;
          }
          if (strnicmp(&commandLine[j],  "G2", 2) == 0)
          {
              if (v.NumVDrivers > 1) currentVDriver = 1;
          }
          if (strnicmp(&commandLine[j],  "T1", 2) == 0)
          {
              firstTest = 0;
              lastTest = 0;  
          }
          if (strnicmp(&commandLine[j],  "T2", 2) == 0)
          {
              firstTest = 1;
              lastTest = 1;  
          }
          if (strnicmp(&commandLine[j],  "T3", 2) == 0)
          {
              firstTest = 2;
              lastTest = 2;  
          }
          if (strnicmp(&commandLine[j],  "T4", 2) == 0)
          {
              firstTest = 3;
              lastTest = 3;  
          }
          if (strnicmp(&commandLine[j],  "T5", 2) == 0)
          {
              firstTest = 4;
              lastTest = 4;  
          }
          if (strnicmp(&commandLine[j],  "SAVEHR", 6) == 0)
          {
              saveHeadings = TRUE;
              saveResults = TRUE;
              saveEnd = FALSE;
              autoRun = TRUE;
          }
          if (strnicmp(&commandLine[j],  "SAVER", 5) == 0)
          {
              saveHeadings = FALSE;
              saveResults = TRUE;
              saveEnd = FALSE;
              autoRun = TRUE;
          }
          if (strnicmp(&commandLine[j],  "SAVERE", 6) == 0)
          {
              saveHeadings = FALSE;
              saveResults = TRUE;
              saveEnd = TRUE;
              autoRun = TRUE;
          }
       }
     
    }

    strcpy(testTitle[0], "BltFast VRAM ");
    strcpy(testTitle[1], "Blt Stretch  ");
    strcpy(testTitle[2], "Blt Fill     ");
    strcpy(testTitle[3], "Blt VSYNCH   ");     
    strcpy(testTitle[4], "Blt CPU RAM  ");
    strcpy(testTitle[5], "10000 Lines  ");
     
    /*
     * display window
     */
    ShowWindow( hwnd, SW_SHOWMAXIMIZED ); 
    UpdateWindow( hwnd );

    if (!autoRun)
    {
         sprintf(mbuff, "Command: %s\n\n"
                      "Test at %d x %d x %d bits\n\n"
                      
                      "At end, results should be\n"
                      "in file %s\n\n"
                      "Press OK to run", 
                   commandLine, screenWide, screenHigh, bitsaPixel, logFile);
        if (MessageBox(NULL, mbuff, "VideoDD Direct Draw Speed Tests",
                    MB_ICONINFORMATION | MB_OKCANCEL) == IDCANCEL)
        {
            return FALSE;
        }
    }
    return TRUE;
                        
}  // InitApp



// WinMain - initialization, message loop

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPSTR cmdline, int cmdshow )
{
    MSG         msg;
    
    hPrevInstance = hPrevInstance;

    if (!InitApp(hInstance, cmdline, cmdshow))
         return FALSE;

    GetMessage( &msg, NULL, 0, 0 );

    if (!setUpTests(mainHWND))
    {
        bQuit = TRUE;        
    }
    testPass = firstTest;
    
    while(!bQuit)
    {
                    
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ))
        {
            
           if (msg.message == WM_QUIT)
           {
                bQuit = TRUE;
                break;
           }
   
           if( !GetMessage( &msg, NULL, 0, 0 ) ) return (int)msg.wParam;
           TranslateMessage(&msg);
           DispatchMessage(&msg);
        }
        else if (testPass > lastTest)
        {
            EndTests();
            bQuit = TRUE;
        }
        else if (testPass < lastTest+1)
        {
            ShowCursor(FALSE);
            if (!runTests())
            {
                if( lpDD != NULL ) finiObjects();
            }
            testPass = testPass + 1;
            ShowCursor(TRUE);
        }
        else
        {
               // make sure we go to sleep if we have
               // nothing else to do
               WaitMessage();
        }
    }
    return (int)msg.wParam;
     
} // WinMain


BOOL setUp( HWND hwnd )
{
    DDSURFACEDESC       ddsd;
    HRESULT             ddrval;
    
    /*
     * create the main DirectDraw object
     */


    ddrval = DirectDrawCreate(v.lpGUID[currentVDriver], &lpDD, NULL );
    if( ddrval != DD_OK )
    {
        sprintf(failMessage, "DirectDrawCreate FAILED");
        return initFail(hwnd);
    }

    
    // Get exclusive mode
    ddrval = lpDD->SetCooperativeLevel( hwnd, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN );
    if( ddrval != DD_OK )
    {
        sprintf(failMessage, "SetCooperativeLevel FAILED");
        if (ddrval == DDERR_HWNDALREADYSET) strcat(failMessage, " DDERR_HWNDALREADYSET");
        if (ddrval == DDERR_INVALIDPARAMS ) strcat(failMessage, " DDERR_INVALIDPARAMS ");
        if (ddrval == DDERR_HWNDSUBCLASSED ) strcat(failMessage, " DDERR_HWNDSUBCLASSED");
        if (ddrval == DDERR_INVALIDOBJECT ) strcat(failMessage, " DDERR_INVALIDOBJECT ");
        if (ddrval == DDERR_EXCLUSIVEMODEALREADYSET ) strcat(failMessage, " DDERR_EXCLUSIVEMODEALREADYSET");
        if (ddrval == DDERR_OUTOFMEMORY) strcat(failMessage, " DDERR_OUTOFMEMORY ");
        return initFail(hwnd);
    }

    // Create the primary surface with no back buffer
    ddsd.dwSize = sizeof( ddsd );
   
    ddsd.dwFlags = DDSD_CAPS; 
    ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE |
                          DDSCAPS_3DDEVICE;

          
    ddrval = lpDD->CreateSurface( &ddsd, &lpDDSPrimary, NULL );
    if( ddrval != DD_OK )
    {
        sprintf(failMessage, "CreateSurface failed");
        return initFail(hwnd);
    }
   
    // create and set the palette
    lpDDPal = DDLoadPalette(lpDD, szBitmap1);

    if (lpDDPal)
    {
        lpDDSPrimary->SetPalette(lpDDPal);
    }
    else
    {
        sprintf(failMessage, "DDLoadPalette failed");
        return initFail(hwnd);
    }
        

    // Create the offscreen surface, by loading our bitmap.
    lpDDSOne = DDLoadBitmap(lpDD, szBitmap1, 0, 0);

    if( lpDDSOne == NULL )
    {
        sprintf(failMessage, "DDLoadBitmap 1 FAILED");
        return initFail(hwnd);
    }

    lpDDSTwo = DDLoadBitmap(lpDD, szBitmap2, 0, 0);

    if( lpDDSTwo == NULL )
    {
        sprintf(failMessage, "DDLoadBitmap 2 FAILED");
        return initFail(hwnd);
    }

    //  Create the large memory area
    
    ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
    ddsd.ddsCaps.dwCaps = DDSCAPS_SYSTEMMEMORY;
    ddsd.dwHeight = bigSize;
    ddsd.dwWidth =  bigSize;
    ddrval = lpDD->CreateSurface( &ddsd, &lpDDSBig, NULL );
    if( ddrval != DD_OK )
    {
        sprintf(failMessage, "Creating system memory bitmap FAILED at %d MB",
                                                (int)(mbytes*bitsaPixel/8 + 0.5));
        return initFail(hwnd);
    }


    // Set the color key for this bitmap (black)

  // U/S ddrval = lpDDSPrimary->SetColorKey(DDCKEY_SRCBLT,  LPDDCOLORKEY); 
    
    DDSetColorKey(lpDDSOne, RGB(0,0,0));
    DDSetColorKey(lpDDSTwo, RGB(0,0,0));
    
    winRect.left   = 0;
    winRect.right  = screenWide;
    winRect.top    = 0;
    winRect.bottom = screenHigh;
 
    memset(&ddbl, 0, sizeof(ddbl));
    ddbl.dwSize = sizeof(DDBLTFX);
       
    return TRUE ;
    
}   // ******** setUp

BOOL  waitSynch()
{
    HRESULT     ddrval;
    
    sprintf (testres, "    WaitForVerticalBlank did not work     ");
    ddrval = lpDD->WaitForVerticalBlank(DDWAITVB_BLOCKBEGIN, NULL);
    return ddrval;
}


BOOL setUpTests(HWND hwnd)
{
       int j;
       HBRUSH hbr;
       HDC  hdc;
       
       escPress = FALSE;
                     
       outfile = fopen(logFile,"a+");  
       if (outfile == NULL)
       {
           if (MessageBox(hwnd, "Cannot open results file - Exit ?", "Error",
                MB_ICONQUESTION | MB_YESNO) == IDYES) exit(0);
       }
       local_time();
       if (saveHeadings)
       {
           fprintf(outfile, "\n\n      ****************************************************************\n\n");   
           fprintf(outfile, "  Direct Draw Test %s, %s", version, timeday);
           fprintf(outfile, "                 Copyright (C) 1998 - 2005, Roy Longbottom\n\n");
           fprintf(outfile, "                     %d seconds tests, %s,\n\n",
                                       runTime, v.VDriverDesc[currentVDriver]);
           fprintf(outfile, "              ");
           fprintf(outfile, "Millions of Pixels Per Second ......... Frames Per Second .....\n"); 
           fprintf(outfile, "  Resolution  ");
           fprintf(outfile, "bltfast blt_and bltfill bltrect bltfast  vsynch bltfast bltfast\n");
           fprintf(outfile, "              ");
           fprintf(outfile, "vid_ram stretch rectngl +vsynch cpu_ram refresh vid_ram cpu_ram\n\n");
           fflush(outfile);
       }
       for(j=0; j<maxTests; j++)
       {
           mpixPerSec[j] = 0.0;
           framesPerSec[j] = 0.0;
       }
       keyPress = GetAsyncKeyState(VK_ESCAPE);

       ShowCursor(FALSE);
       bigSize = screenWide;
       if(!setUp(hwnd ))
       {
            finiObjects();
            return FALSE;
       }
       if (lpDDSPrimary->GetDC(&hdc) == DD_OK)
       {
           hbr = CreateSolidBrush(RGB(0, 0, 0));  
           SelectObject(hdc, hbr);
           Rectangle(hdc, 0, 0, screenWide, screenHigh);
           DeleteObject(hbr);

           SetBkColor(hdc, RGB(0, 0, 0));
           SetTextColor(hdc, RGB(255, 0, 0)); 
           sprintf (testres, "Test Starting %dx%dx%d bits                                                            ",
                                   screenWide, screenHigh, bitsaPixel);
           TextOut(hdc, 10,  1, testres, lstrlen(testres));
           lpDDSPrimary->ReleaseDC(hdc);
       }      
       return TRUE;

}  // setUpTests

BOOL EndTests()
{
    int i;
    
    if (saveResults)
    {
        fprintf (outfile, "%5d %4d %2d", screenWide, screenHigh, bitsaPixel);
        fprintf (outfile, "%8.1f", mpixPerSec[0]);          
        fprintf (outfile, "%8.1f", mpixPerSec[1]);          
        fprintf (outfile, "%8.1f", mpixPerSec[2]);          
        fprintf (outfile, "%8.1f", mpixPerSec[3]);                 
        fprintf (outfile, "%8.1f", mpixPerSec[4]);
        fprintf (outfile, "%8.1f", framesPerSec[3]);
        fprintf (outfile, "%8.1f", framesPerSec[0]);
        fprintf (outfile, "%8.1f", framesPerSec[4]);
        fprintf (outfile, "\n");                
    }
    if (escPress) fprintf(outfile, "\n ESCAPE PRESSED\n\n");

   if (saveEnd)
   {
       getDetails();
       fprintf(outfile, "\n");
       for (i=0; i<9; i++)
       {
           fprintf(outfile, "%s\n", configdata[i]);
       }
       local_time();
       fprintf (outfile, "\n          Run Time Each Test %d seconds, end at %s \n", runTime, timeday);
   }
   fflush (outfile);
   fclose(outfile);
   finiObjects();
   return TRUE;

} // EndTests   



BOOL FAR PASCAL DDEnumCallback( LPGUID lpGUID, 
                                LPSTR lpDriverDesc,
                                LPSTR lpDriverName, 
                                LPVOID lpContext )
{
   LPDIRECTDRAW lpDD;
   DDCAPS       DDcaps, HELcaps;
   lpContext = lpContext;
   
     
   // try and create a DD device using the specified GUID
   
   if ( DirectDrawCreate( lpGUID, &lpDD, NULL ) != DD_OK )
   {
      // failed, so ignore this device
      return DDENUMRET_OK;
   }
 
   
   // get caps of this DD driver. If it fails,  on to the next
   // driver
   
   memset( &DDcaps, 0, sizeof( DDcaps ) );
   DDcaps.dwSize = sizeof( DDcaps );
   memset( &HELcaps, 0, sizeof( HELcaps ) );
   HELcaps.dwSize = sizeof( HELcaps );
   if ( lpDD->GetCaps( &DDcaps, &HELcaps ) != DD_OK )
   {
      RELEASE(lpDD);
      return DDENUMRET_OK;
   }
   
   // device is valid! Now store relevant information in our 
   // device list

   if ( lpGUID == NULL )
   {
       v.lpGUID[v.NumVDrivers] = 0;
   }
   else
   {
      v.lpGUID[v.NumVDrivers] = &v.VDriverGUID[v.NumVDrivers];
      memcpy(&v.VDriverGUID[v.NumVDrivers], lpGUID, sizeof(GUID));
   }
   lstrcpy(&v.VDriverName[v.NumVDrivers][0], lpDriverName);
   lstrcpy(&v.VDriverDesc[v.NumVDrivers][0], lpDriverDesc);
   v.NumVDrivers = v.NumVDrivers + 1;

  
   RELEASE(lpDD);

   if ( v.NumVDrivers == 5 )
      return DDENUMRET_CANCEL;

   return DDENUMRET_OK;

} // DDEnumCallback

BOOL Test1()
{
        HRESULT     ddrval;
        int   i, j, k;
        HDC   hdc;

        noGo = FALSE;
        for(i=0; i<runPasses; i++)
        {
              oddCol = oddColStart;
              for(k=0; k<picHigh*64; k=k+64)
              {
                if (oddCol)
                {
                   oddCol = FALSE;
                }
                else
                {
                   oddCol = TRUE;
                }                      
                for(j=0; j<picWide*64; j=j+128)
                {
                    while( 1 )
                    {
                        // ************ Test 1  testPass 0   BltFast offscreen no back buffer
                        if (oddCol)
                        {
                             ddrval = lpDDSPrimary->BltFast( picLeft + j, picTop + k, lpDDSOne,
                                      &rcRect, DDBLTFAST_NOCOLORKEY | DDBLTFAST_WAIT);
                             if( ddrval == DD_OK )
                             {
                                 ddrval = lpDDSPrimary->BltFast( picLeft + j + 64, picTop + k, lpDDSOne,
                                      &rcRect2, DDBLTFAST_NOCOLORKEY | DDBLTFAST_WAIT);
                             }

                        }
                        else
                        {
                             ddrval = lpDDSPrimary->BltFast( picLeft + j, picTop + k, lpDDSOne,
                                      &rcRect2, DDBLTFAST_NOCOLORKEY | DDBLTFAST_WAIT);
                             if( ddrval == DD_OK )
                             {
                                 ddrval = lpDDSPrimary->BltFast( picLeft + j + 64, picTop + k, lpDDSOne,
                                      &rcRect, DDBLTFAST_NOCOLORKEY | DDBLTFAST_WAIT);
                             }

                        }
                        if( ddrval == DD_OK )
                        {
                            break;
                        }
                        if( ddrval != DDERR_SURFACELOST )
                        {
                            ddrval = restoreAll();
                            if( ddrval != DD_OK )
                            {
                                sprintf(failMessage, "restoreAll 2 FAILED");
                                return initFail(mainHWND);
                            }
                        }
                        if( ddrval != DDERR_WASSTILLDRAWING )
                        {
                            noGo = TRUE;
                            return TRUE;
                        }
                    }
                    if(ddrval != DD_OK)
                    {
                        sprintf(failMessage, "ddrval != DD_OK 2 FAILURE");
                        return initFail(mainHWND);
                    }
                }
              }             
              if (waitForSynch)
              {
                  if (waitSynch() != DD_OK )
                  {
                      sprintf(failMessage, "waitSynch 2 FAILURE");
                      return initFail(mainHWND);
                  }
              }
        }
        if (lpDDSPrimary->GetDC(&hdc) == DD_OK)
        {
           gotPixel = gotPixel | GetPixel(hdc, 2, 1);
           lpDDSPrimary->ReleaseDC(hdc);
        }
        return TRUE;
} // Test1

BOOL Test2()
{
        HRESULT     ddrval;
        int   i, ii;
        HDC   hdc;

        noGo = FALSE;
        j1 = j1 + 50;
        if (j1 > screenWide - k1 - 130) j1 = 0;
        rcRect2.top    = 30;
        rcRect2.bottom = 158;
        rcRect2.left   = 0 + j1;
        rcRect2.right  = 128 + j1;
        
        for(i=0; i<k1; i=i+k1inc)
        {
             rcRect2.left  = rcRect2.left;
             rcRect2.right = rcRect2.right + k1inc;
             rcRect2.bottom = rcRect2.bottom + k1inc;
             for(ii=0; ii<runPasses; ii++)
             {
                while( 1 )       
                {
                    
                    ddrval = lpDDSPrimary->Blt( &rcRect2, lpDDSOne, &rcRect, DDBLT_WAIT , &ddbl); 
                    if( ddrval == DD_OK )
                    {
                        break;
                    }
                    if( ddrval == DDERR_SURFACELOST )
                    {
                        ddrval = restoreAll();
                        if( ddrval != DD_OK )
                        {
                            sprintf(failMessage, "restoreAll 3 FAILED");
                            return initFail(mainHWND);
                        }
                    }
                    if( ddrval != DDERR_WASSTILLDRAWING )
                    {
                        noGo = TRUE;
                        return TRUE;
                    }
                }
                if(ddrval != DD_OK)
                {
                    sprintf(failMessage, "ddrval != DD_OK 3 FAILURE");
                    return initFail(mainHWND);
                }
                testCount = testCount + (rcRect2.right - rcRect2.left)*(rcRect2.bottom - rcRect2.top);
                
                if (waitForSynch)
                {
                    if (waitSynch() != DD_OK )
                    {
                        sprintf(failMessage, "waitSynch 3 FAILURE");
                        return initFail(mainHWND);
                    }
                }          
             }
        }
        if (lpDDSPrimary->GetDC(&hdc) == DD_OK)
        {
           gotPixel = gotPixel | GetPixel(hdc, 2, 1);
           lpDDSPrimary->ReleaseDC(hdc);
        }
        return TRUE;
} // Test2

BOOL Test3and4()
{
        HRESULT     ddrval;
        int   i, j, k, ii;
        HDC   hdc;
        int      rd[2];
        int      gr[2];
        int      bl[2];
            
       noGo = FALSE;
       bl[0] = i1;
       bl[1] = i2;       
       rd[0] = i1;
       rd[1] = i2;
       gr[0] = i1;
       gr[1] = i2;    
       i1 = i1 - i3;
       i2 = i2 + i3;
       if (i2 > 255)
       {
           i2 = 255;
           i1 = 0;
           i3 = - i3;
       }
       if (i1 > 255)
       {
           i1 = 255;
           i2 = 0;
           i3 = - i3;
       }
       i4 = i4 + 20;
       if (i4 > 220) i4 = 0;
          
       for(ii=0; ii<runPasses; ii++)
       {
         rcScreen.bottom = picTop;
         rcScreen.top    = picTop - sceeenInc;
         for(i=0; i<2; i++)
         {
           for (j=0; j<2; j++)
           {
              for (k=0; k<2; k++)
              {
                rcScreen.top = rcScreen.top + sceeenInc;
                rcScreen.bottom = rcScreen.bottom + sceeenInc;
                if (rcScreen.bottom > screenHigh) rcScreen.bottom = screenHigh;  
                while( 1 )       
                {
                    ddbl.dwFillColor = RGB(bl[i],gr[j],rd[k]);
                    if(bitsaPixel == 8) ddbl.dwFillColor = PALETTEINDEX(i+j*8+k*16+i4);
                    ddrval = lpDDSPrimary->Blt( &rcScreen, NULL, NULL , DDBLT_COLORFILL | DDBLT_WAIT, &ddbl); 
    
                    if( ddrval == DD_OK )
                    {
                        break;
                    }
                    if( ddrval == DDERR_SURFACELOST )
                    {
                        ddrval = restoreAll();
                        if( ddrval != DD_OK )
                        {
                            sprintf(failMessage, "restoreAll 4 FAILED");
                            return initFail(mainHWND);
                        }
                    }
                    if( ddrval != DDERR_WASSTILLDRAWING )
                    {
                        noGo = TRUE;
                        return TRUE;
                    }
                }
                if(ddrval != DD_OK)
                {
                    sprintf(failMessage, "ddrval != DD_OK 4 FAILURE");
                    return initFail(mainHWND);
                }
              }
           }
         }
         // ************ Test 4  testPass 3 Blt WaitForVerticalBlank  
        
         if( testPass == 3)
         {
             if (waitSynch() != DD_OK )
             {
                sprintf(failMessage, "waitSynch 4 FAILURE");
                return initFail(mainHWND);
             }
         }
         else
         {
           if (waitForSynch)
           {
              if (waitSynch() != DD_OK )
              {
                  sprintf(failMessage, "waitSynch 4b FAILURE");
                  return initFail(mainHWND);
              }
           }
         }

         testCount = testCount + 1.0; 
       }
       if (lpDDSPrimary->GetDC(&hdc) == DD_OK)
       {
           gotPixel = gotPixel | GetPixel(hdc, 2, 1);
           lpDDSPrimary->ReleaseDC(hdc);
       }
       return TRUE;

} // Test3and4

BOOL Test5()
{
       HRESULT     ddrval;
       int   ii;
       HDC   hdc;

       noGo = FALSE;
       for(ii=0; ii<runPasses; ii++)
       {
          rcRect.top = rt;
          rcRect.bottom = rb;
          while( 1 )
          {
              ddrval = lpDDSPrimary->BltFast( 0, picTop, lpDDSBig,
                         &rcRect, DDBLTFAST_NOCOLORKEY | DDBLTFAST_WAIT);

              if( ddrval == DD_OK )
              {
                  break;
              }
              if( ddrval != DDERR_SURFACELOST )
              {
                  ddrval = restoreAll();
                  if( ddrval != DD_OK )
                  {
                     sprintf(failMessage, "restoreAll 5 FAILED");
                     return initFail(mainHWND);
                  }
              }
              if( ddrval != DDERR_WASSTILLDRAWING )
              {
                    noGo = TRUE;
                    return TRUE;
              }
          }
          if(ddrval != DD_OK)
          {
              sprintf(failMessage, "ddrval != DD_OK 5 FAILURE");
              return initFail(mainHWND);
          }

          if (!reverse)
          {
               rt = rt + i4;
               rb = rb + i4;
               if (rt > ri)
               {
                   reverse = TRUE;
                   rt = rt - i4;
                   rb = rb - i4;
               }
          }
          else
          {
               rt = rt - i4;
               rb = rb - i4;
               if (rt < 1)
               {
                   reverse = FALSE;
                   rt = 0;
                   rb = screenHigh - picTop;
               }
          }

          if (waitForSynch)
          {
               if (waitSynch() != DD_OK )
               {
                    sprintf(failMessage, "waitSynch 5 FAILURE");
                    return initFail(mainHWND);
               }
          }                           
          testCount = testCount + 1.0;    
       }
       if (lpDDSPrimary->GetDC(&hdc) == DD_OK)
       {
           gotPixel = gotPixel | GetPixel(hdc, 2, 1);
           lpDDSPrimary->ReleaseDC(hdc);
       }
       return TRUE;

} // Test5

void local_time()
{

    time_t time_of_day;
    time_of_day = time( NULL ); 
    strcpy (timeday, ctime(&time_of_day)); // timeday = date and time
    return;
}

